#!/usr/bin/env python

""" MultiQC submodule to parse output from RSeQC junction_annotation.py
http://rseqc.sourceforge.net/#junction-annotation-py """

from collections import OrderedDict
import logging
import re

from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find RSeQC junction_annotation reports and parse their data """

    # Set up vars
    self.junction_annotation_data = dict()
    regexes = {
        'total_splicing_events': r"^Total splicing  Events:\s*(\d+)$",
        'known_splicing_events': r"^Known Splicing Events:\s*(\d+)$",
        'partial_novel_splicing_events': r"^Partial Novel Splicing Events:\s*(\d+)$",
        'novel_splicing_events': r"^Novel Splicing Events:\s*(\d+)$",
        'total_splicing_junctions': r"^Total splicing  Junctions:\s*(\d+)$",
        'known_splicing_junctions': r"^Known Splicing Junctions:\s*(\d+)$",
        'partial_novel_splicing_junctions': r"^Partial Novel Splicing Junctions:\s*(\d+)$",
        'novel_splicing_junctions': r"^Novel Splicing Junctions:\s*(\d+)$",
    }

    # Go through files and parse data using regexes
    for f in self.find_log_files('rseqc/junction_annotation'):
        d = dict()
        for k, r in regexes.items():
            r_search = re.search(r, f['f'], re.MULTILINE)
            if r_search:
                d[k] = int(r_search.group(1))

        # Calculate some percentages
        if 'total_splicing_events' in d:
            t = float(d['total_splicing_events'])
            if 'known_splicing_events' in d:
                d['known_splicing_events_pct'] = (float(d['known_splicing_events']) / t)*100.0
            if 'partial_novel_splicing_events' in d:
                d['partial_novel_splicing_events_pct'] = (float(d['partial_novel_splicing_events']) / t)*100.0
            if 'novel_splicing_events' in d:
                d['novel_splicing_events_pct'] = (float(d['novel_splicing_events']) / t)*100.0
        if 'total_splicing_junctions' in d:
            t = float(d['total_splicing_junctions'])
            if 'known_splicing_junctions' in d:
                d['known_splicing_junctions_pct'] = (float(d['known_splicing_junctions']) / t)*100.0
            if 'partial_novel_splicing_junctions' in d:
                d['partial_novel_splicing_junctions_pct'] = (float(d['partial_novel_splicing_junctions']) / t)*100.0
            if 'novel_splicing_junctions' in d:
                d['novel_splicing_junctions_pct'] = (float(d['novel_splicing_junctions']) / t)*100.0

        if len(d) > 0:
            if f['s_name'] in self.junction_annotation_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
            self.add_data_source(f, section='junction_annotation')
            self.junction_annotation_data[f['s_name']] = d

    # Filter to strip out ignored sample names
    self.junction_annotation_data = self.ignore_samples(self.junction_annotation_data)

    if len(self.junction_annotation_data) > 0:

        # Write to file
        self.write_data_file(self.junction_annotation_data, 'multiqc_rseqc_junction_annotation')

        # Plot junction annotations
        keys = [OrderedDict(), OrderedDict()]
        keys[0]['known_splicing_junctions'] = { 'name': 'Known Splicing Junctions' }
        keys[0]['partial_novel_splicing_junctions'] = { 'name': 'Partial Novel Splicing Junctions' }
        keys[0]['novel_splicing_junctions'] = { 'name': 'Novel Splicing Junctions' }
        keys[1]['known_splicing_events'] = { 'name': 'Known Splicing Events' }
        keys[1]['partial_novel_splicing_events'] = { 'name': 'Partial Novel Splicing Events' }
        keys[1]['novel_splicing_events'] = { 'name': 'Novel Splicing Events' }

        pconfig = {
            'id': 'rseqc_junction_annotation_junctions_plot',
            'title': 'RSeQC: Splicing Junctions',
            'ylab': '% Junctions',
            'cpswitch_c_active': False,
            'data_labels': [ 'Junctions', 'Events' ]
        }
        self.add_section (
            name = 'Junction Annotation',
            anchor = 'rseqc_junction_annotation',
            description = '<a href="http://rseqc.sourceforge.net/#junction-annotation-py" target="_blank">Junction annotation</a>' \
                " compares detected splice junctions to" \
                " a reference gene model. An RNA read can be spliced 2" \
                " or more times, each time is called a splicing event.",
            plot = bargraph.plot([self.junction_annotation_data, self.junction_annotation_data], keys, pconfig)
        )

    # Return number of samples found
    return len(self.junction_annotation_data)




# !/usr/bin/env python

""" MultiQC module to parse output from DeDup """

from __future__ import print_function
from collections import OrderedDict
import logging
import json

from multiqc.plots import bargraph
from multiqc.utils import config
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ DeDup module """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='DeDup', anchor='dedup',
        href="http://www.github.com/apeltzer/DeDup",
        info="is a tool for duplicate removal for merged/collapsed reads in ancient DNA analysis.")

        # Find and load any DeDup reports
        self.dedup_data = dict()

        for f in self.find_log_files('dedup',filehandles=True):
            try:
                self.parseJSON(f)
            except KeyError:
                logging.warning("Error loading file {}".format(f['fn']))

        # Filter to strip out ignored sample names
        self.dedup_data = self.ignore_samples(self.dedup_data)

        if len(self.dedup_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.dedup_data)))

        # Write parsed report data to a file
        self.write_data_file(self.dedup_data, 'multiqc_dedup')

        # Basic Stats Table
        self.dedup_general_stats_table()

        # Alignment Rate Plot
        self.add_section(
            description = 'This plot shows read categories that were either not removed (unique reads) or removed (duplicates).',
            plot = self.dedup_alignment_plot()
        )

    # Parse our nice little JSON file
    def parseJSON(self, f):
        """ Parse the JSON output from DeDup and save the summary statistics """
        try:
            parsed_json = json.load(f['f'])
            # Check for Keys existing
            if 'metrics' not in parsed_json or 'metadata' not in parsed_json:
                log.debug("DeDup JSON missing essential keys - skipping sample: '{}'".format(f['fn']))
                return None
        except JSONDecodeError as e:
            log.debug("Could not parse DeDup JSON: '{}'".format(f['fn']))
            log.debug(e)
            return None

        # Get sample name from JSON first
        s_name = self.clean_s_name(parsed_json['metadata']['sample_name'], f['root'])
        self.add_data_source(f, s_name)

        metrics_dict = parsed_json['metrics']

        for k in metrics_dict:
            metrics_dict[k] = float(metrics_dict[k])

        # Compute (not) removed _mapped_ reads from given values as dedup only affects mapped reads
        # Keep legacy behaviour in case "mapped_reads" cannot be found for <= v0.12.6
        if 'mapped_reads' in metrics_dict:
            metrics_dict['mapped_after_dedup'] = (
                metrics_dict['mapped_reads']
                    - metrics_dict['reverse_removed']
                    - metrics_dict['forward_removed']
                    - metrics_dict['merged_removed']
            )
        else:
            metrics_dict['not_removed'] = (
                metrics_dict['total_reads']
                    - metrics_dict['reverse_removed']
                    - metrics_dict['forward_removed']
                    - metrics_dict['merged_removed']
            )
        metrics_dict['reads_removed'] = (
            metrics_dict['reverse_removed']
            + metrics_dict['forward_removed']
            + metrics_dict['merged_removed']
        )

        # Add all in the main data_table
        self.dedup_data[s_name] = metrics_dict


    def dedup_general_stats_table(self):
        """ Take the parsed stats from the DeDup report and add it to the
        basic stats table at the top of the report """

        ancient_read_count_prefix = getattr(config, 'ancient_read_count_prefix', 'K')
        ancient_read_count_desc = getattr(config, 'ancient_read_count_desc', 'thousands')
        ancient_read_count_multiplier = getattr(config, 'ancient_read_count_multiplier',  0.001)

        headers = OrderedDict()
        headers['dup_rate'] = {
            'title': 'Duplication Rate',
            'description': 'Percentage of reads categorised as a technical duplicate',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'OrRd',
            'format': '{:,.0f}',
            'modify': lambda x: x * 100.0
        }
        headers['clusterfactor'] = {
            'title': 'ClusterFactor',
            'description': 'CF~1 means high library complexity. Large CF means not worth sequencing deeper.',
            'min': 1,
            'max': 100,
            'scale': 'OrRd',
            'format': '{:,.2f}',
        }
        headers['reads_removed'] = {
            'title': '{} Reads Removed'.format(ancient_read_count_prefix),
            'description': 'Non-unique reads removed after deduplication ({})'.format(ancient_read_count_desc),
            'modify': lambda x: x * ancient_read_count_multiplier,
            'shared_key': 'read_count',
            'min': 0,
            'hidden': True
        }
        headers['mapped_after_dedup'] = {
            'title': '{} Post-DeDup Mapped Reads'.format(ancient_read_count_prefix),
            'description': 'Unique mapping reads after deduplication ({})'.format(ancient_read_count_desc),
            'modify': lambda x: x * ancient_read_count_multiplier,
            'shared_key': 'read_count',
            'min': 0,
        }
        self.general_stats_addcols(self.dedup_data, headers)

    def dedup_alignment_plot (self):
        """ Make the HighCharts HTML to plot the duplication rates """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['mapped_after_dedup'] = { 'name': 'Unique Retained' }
        keys['not_removed'] = { 'name': 'Not Removed' }
        keys['reverse_removed'] = { 'name': 'Reverse Removed' }
        keys['forward_removed'] = { 'name': 'Forward Removed' }
        keys['merged_removed'] = { 'name': 'Merged Removed' }

        # Config for the plot
        config = {
            'id': 'dedup_rates',
            'title': 'DeDup: Deduplicated Reads',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        return bargraph.plot(self.dedup_data, keys, config)



from itertools import chain

from multiqc.plots import linegraph

def plot_indelhist(samples, file_type, **plot_args):
    """ Create line graph plot of histogram data for BBMap 'indelhist' output.

    The 'samples' parameter could be from the bbmap mod_data dictionary:
    samples = bbmap.MultiqcModule.mod_data[file_type]
    """

    all_x = set()
    for item in sorted(chain(*[samples[sample]['data'].items()
                                for sample in samples])):
        all_x.add(item[0])


    columns_to_plot = {
        'Deletions': {
            0: 'Count',
        },
        'Insertions': {
            1: 'Count',
        }
    }

    plot_data = []
    for column_type in columns_to_plot:
        plot_data.append(
            {
                sample+'.'+column_name: {
                    x: samples[sample]['data'][x][column] if x in samples[sample]['data'] else 0
                    for x in all_x
                }
                for sample in samples
                for column, column_name in columns_to_plot[column_type].items()
            }
        )

    plot_params = {
            'id': 'bbmap-' + file_type + '_plot',
            'title': 'BBTools: ' + plot_args['plot_title'],
            'xlab': 'Indel size',
            'ylab': 'Insertion count',
            'data_labels': [
                {'name': 'Insertions', 'ylab': 'Insertion count'},
                {'name': 'Deletions', 'ylab': 'Deletion count'},
            ]

    }
    plot_params.update(plot_args['plot_params'])
    plot = linegraph.plot(
        plot_data,
        plot_params
    )

    return plot



#!/usr/bin/env python

""" MultiQC module to parse output from BUSCO """

from __future__ import print_function
from collections import OrderedDict
import logging
import re
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ BUSCO module """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='BUSCO', anchor='busco',
        href="http://busco.ezlab.org/",
        info="assesses genome assembly and annotation completeness with Benchmarking Universal Single-Copy Orthologs.")

        # Keys and strings, used for parsing and for plot
        self.busco_keys = {
            'complete': 'Complete BUSCOs',
            'complete_single_copy': 'Complete and single-copy BUSCOs',
            'complete_duplicated': 'Complete and duplicated BUSCOs',
            'fragmented': 'Fragmented BUSCOs',
            'missing': 'Missing BUSCOs',
            'total': 'Total BUSCO groups searched'
        }

        # Find and load any BUSCO reports
        self.busco_data = dict()
        for f in self.find_log_files('busco'):
            self.parse_busco_log(f)

        # Filter to strip out ignored sample names
        self.busco_data = self.ignore_samples(self.busco_data)

        if len(self.busco_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.busco_data)))

        # Write parsed report data to a file
        self.write_data_file(self.busco_data, 'multiqc_busco')

        # One Alignment Rate Plot per lineage
        lineages = set([ self.busco_data[s_name].get('lineage_dataset') for s_name in self.busco_data.keys() ])
        for lin in lineages:
            self.add_section (
                name = 'Lineage Assessment' if lin is None else 'Lineage: {}'.format(lin),
                anchor = 'busco-lineage-{}'.format(re.sub('\W+', '_', str(lin))),
                plot = self.busco_plot(lin)
            )

    def parse_busco_log(self, f):
        parsed_data = {}
        for l in f['f'].splitlines():
            for key, string in self.busco_keys.items():
                if string in l:
                    s = l.strip().split("\t")
                    parsed_data[key] = float(s[0])
            if 'The lineage dataset is:' in l:
                s = l.replace('# The lineage dataset is: ', '').split(' (Creation date:', 1)
                parsed_data['lineage_dataset'] = str(s[0])

        if len(parsed_data) > 0:
            self.busco_data[f['s_name']] = parsed_data

    def busco_plot (self, lin):
        """ Make the HighCharts HTML for the BUSCO plot for a particular lineage """

        data = {}
        for s_name in self.busco_data:
            if self.busco_data[s_name].get('lineage_dataset') == lin:
                data[s_name] = self.busco_data[s_name]

        plot_keys = ['complete_single_copy','complete_duplicated','fragmented','missing']
        plot_cols = ['#7CB5EC', '#434348', '#F7A35C', '#FF3C50']
        keys = OrderedDict()
        for k, col in zip(plot_keys, plot_cols):
            keys[k] = {'name': self.busco_keys[k], 'color': col}

        # Config for the plot
        config = {
            'id': 'busco_plot_{}'.format(re.sub('\W+', '_', str(lin))),
            'title': 'BUSCO: Assessment Results' if lin is None else 'BUSCO Assessment Results: {}'.format(lin),
            'ylab': '# BUSCOs',
            'cpswitch_counts_label': 'Number of BUSCOs'
        }

        return bargraph.plot(data, keys, config)



#!/usr/bin/env python

""" MultiQC module to parse output from methylQA """

from __future__ import print_function
from collections import OrderedDict
import logging
import os
import re

from multiqc.plots import linegraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='methylQA', anchor='methylqa',
        target='methylQA', href="http://methylqa.sourceforge.net/",
        info=" - a methylation sequencing data quality assessment tool.")

        # Find and load any methylQA reports
        self.methylqa_data = dict()
        self.methylqa_coverage_counts = dict()
        self.methylqa_coverage_percentages = dict()
        for f in self.find_log_files('methylQA'):
            self.parse_methylqa_logs(f)

        # Filter to strip out ignored sample names
        self.methylqa_data = self.ignore_samples(self.methylqa_data)

        if len(self.methylqa_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.methylqa_data)))

        # Write parsed report data to a file
        self.write_data_file(self.methylqa_data, 'multiqc_methylQA')

        # Basic Stats Table
        self.methylqa_general_stats_table()

        # Alignment bar plot - only one section, so add to the module intro
        self.add_section( plot = self.methylqa_alignment_plot() )


    def parse_methylqa_logs(self, f):

        # Get s_name from first input file if possible
        s_name = f['s_name']
        if f['f'][0].startswith('files provided'):
            s_name = self.clean_s_name(os.path.basename(f['f'][0]))

        parsed_data = {}
        regexes = {
            'mappable_reads': r"uniquely mappable reads \(pair\):\s*(\d+)",
            'quality_failed': r"quality failed mapped reads \(pair\) in the bismark bam:\s*(\d+)",
            'oversized_reads': r"oversized mapped reads \(pair\) in the bismark bam:\s*(\d+)",
            'mapped_bases': r"total base of uniquely mapped reads \(pair\):\s*(\d+)",
            'coverage': r"total base of uniquely mapped reads \(pair\) cover genome base \(\d+\):\s*([\d\.]+)X",
            'CHG_count_meth': r"number of methylated C in CHG context \(was protected\):\s*(\d+)",
            'CHG_count_unmeth': r"number of not methylated C in CHG context \(was converted\):\s*(\d+)",
            'CHG_percent_meth': r"C->T convertion rate in CHG context:\s*([\d\.]+)%",
            'CHH_count_meth': r"number of methylated C in CHH context \(was protected\):\s*(\d+)",
            'CHH_count_unmeth': r"number of not methylated C in CHH context \(was converted\):\s*(\d+)",
            'CHH_percent_meth': r"C->T convertion rate in CHH context:\s*([\d\.]+)%",
            'CG_count_meth': r"number of methylated C in CpG context \(was protected\):\s*(\d+)",
            'CG_count_unmeth': r"number of not methylated C in CpG context \(was converted\):\s*(\d+)",
            'CG_percent_meth': r"C->T convertion rate in CpG context:\s*([\d\.]+)%",
            'CN_count_meth': r"number of methylated C in Unknown context \(was protected\):\s*(\d+)",
            'CN_count_unmeth': r"number of not methylated C in Unknown context \(was converted\):\s*(\d+)",
            'CN_percent_meth': r"C->T convertion rate in Unknown context:\s*([\d\.]+)%",
        }
        for k, r in regexes.items():
            match = re.search(r, f['f'])
            if match:
                parsed_data[k] = float(match.group(1))

        # Parse the coverage histogram
        hist = False
        for l in f['f'].split(os.linesep):
            if hist is not False:
                try:
                    s = l.split()
                    hist['counts'][s[0]] = int(s[1])
                    hist['percentages'][s[0]] = float(s[2])
                except IndexError:
                    break
            if re.search(r"Times covered\s+Count\s+Percent\s+", l):
                hist = { 'counts': OrderedDict(), 'percentages': OrderedDict() }
        if hist is not False and len(hist) > 0:
            self.methylqa_coverage_counts[s_name] = hist['counts']
            self.methylqa_coverage_percentages[s_name] = hist['percentages']

        if len(parsed_data) > 0:
            if s_name in self.methylqa_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
            self.add_data_source(f, s_name)
            self.methylqa_data[s_name] = parsed_data


    def methylqa_general_stats_table(self):
        """ Take the parsed stats from the methylQA report and add it to the
        basic stats table at the top of the report """

        headers = OrderedDict()
        headers['coverage'] = {
            'title': 'Fold Coverage',
            'min': 0,
            'suffix': 'X',
            'scale': 'YlGn'
        }
        self.general_stats_addcols(self.methylqa_data, headers)

    def methylqa_alignment_plot (self):
        """ Make the HighCharts HTML to plot the alignment rates """

        if len(self.methylqa_coverage_counts) == 0:
            return '<div class="alert alert-danger">No histogram data found.</div>'

        pconfig = {
            'id': 'methylqa_coverage',
            'title': 'methylQA: CpG Coverage',
            'categories': True,
            'ylab': 'CpG Counts',
            'xlab': 'Times covered',
            'ymin': 0,
            'data_labels': [
                {'name': 'Counts', 'ylab': 'CpG Counts' },
                {'name': 'Percentages', 'ylab': '% CpGs', 'ymax': 100 }
            ]
        }
        return linegraph.plot([self.methylqa_coverage_counts, self.methylqa_coverage_percentages], pconfig)




#!/usr/bin/env python

"""
Code to initilise the MultiQC logging
"""

import coloredlogs
import logging
import os
import shutil
import sys
import tempfile

from multiqc.utils import config, util_functions

LEVELS = {0: 'INFO', 1: 'DEBUG'}
log_tmp_dir = None
log_tmp_fn = '/dev/null'

def init_log(logger, loglevel=0, no_ansi=False):
    """
    Initializes logging.
    Prints logs to console with level defined by loglevel
    Also prints verbose log to the multiqc data directory if available.
    (multiqc_data/multiqc.log)

    Args:
        loglevel (str): Determines the level of the log output.
    """
    # File for logging
    global log_tmp_dir, log_tmp_fn
    log_tmp_dir = tempfile.mkdtemp()
    log_tmp_fn = os.path.join(log_tmp_dir, 'multiqc.log')

    # Logging templates
    debug_template = '[%(asctime)s] %(name)-50s [%(levelname)-7s]  %(message)s'
    info_template = '[%(levelname)-7s] %(module)15s : %(message)s'

    # Base level setup
    logger.setLevel(getattr(logging, 'DEBUG'))

    # Set up the console logging stream
    console = logging.StreamHandler()
    console.setLevel(getattr(logging, loglevel))
    level_styles = coloredlogs.DEFAULT_LEVEL_STYLES
    level_styles['debug'] = {'faint': True}
    if loglevel == 'DEBUG':
        if no_ansi or not sys.stderr.isatty():
            console.setFormatter(logging.Formatter(debug_template))
        else:
            console.setFormatter(coloredlogs.ColoredFormatter(fmt=debug_template, level_styles=level_styles))
    else:
        if no_ansi or not sys.stderr.isatty():
            console.setFormatter(logging.Formatter(info_template))
        else:
            console.setFormatter(coloredlogs.ColoredFormatter(fmt=info_template, level_styles=level_styles))
    logger.addHandler(console)

    # Now set up the file logging stream if we have a data directory
    file_handler = logging.FileHandler(log_tmp_fn, encoding='utf-8')
    file_handler.setLevel(getattr(logging, 'DEBUG')) # always DEBUG for the file
    file_handler.setFormatter(logging.Formatter(debug_template))
    logger.addHandler(file_handler)

def move_tmp_log(logger):
    """ Move the temporary log file to the MultiQC data directory
    if it exists. """

    try:
        # https://stackoverflow.com/questions/15435652/python-does-not-release-filehandles-to-logfile
        logging.shutdown()
        shutil.copy(log_tmp_fn, os.path.join(config.data_dir, 'multiqc.log'))
        os.remove(log_tmp_fn)
        util_functions.robust_rmtree(log_tmp_dir)
    except (AttributeError, TypeError, IOError):
        pass


def get_log_stream(logger):
    """
    Returns a stream to the root log file.
    If there is no logfile return the stderr log stream

    Returns:
        A stream to the root log file or stderr stream.
    """

    file_stream = None
    log_stream = None
    for handler in logger.handlers:
        if isinstance(handler, logging.FileHandler):
            file_stream = handler.stream
        else:
            log_stream = handler.stream

    if file_stream:
        return file_stream

    return log_stream



#!/usr/bin/env python

""" MultiQC module to parse output from biobambam2 """

from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.modules.picard import MarkDuplicates

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ This module is super weird. The output from this tools is essentially
    identical to Picard MarkDuplicates, so we just hijack that module instead"""

    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='biobambam2', anchor='biobambam2',
        href='https://github.com/gt1/biobambam2',
        info="provides tools for early stage alignment file processing")

        # Set up class objects to hold parsed data
        self.general_stats_headers = OrderedDict()
        self.general_stats_data = dict()
        n = dict()

        n['bamsormadup'] = MarkDuplicates.parse_reports(self,
            log_key='biobambam2/bamsormadup',
            section_name='bamsormadup',
            section_anchor='biobambam2-bamsormadup',
            plot_title='biobambam2: bamsormadup deduplication stats',
            plot_id='biobambam2_bamsormadup_plot',
            data_filename='bamsormadup_bamsormadup')
        if n['bamsormadup'] > 0:
            log.info("Found {} bamsormadup reports".format(n['bamsormadup']))

        # Exit if we didn't find anything
        if sum(n.values()) == 0:
            raise UserWarning

        # Add to the General Stats table (has to be called once per MultiQC module)
        self.general_stats_addcols(self.general_stats_data, self.general_stats_headers)


    # Helper functions
    def multiply_hundred(self, val):
        try:
            val = float(val) * 100
        except ValueError:
            pass
        return val



#!/usr/bin/env python

""" MultiQC module to parse output from Salmon """

from __future__ import print_function
from collections import OrderedDict
import json
import logging
import os

from multiqc.plots import linegraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Salmon', anchor='salmon',
        href='http://combine-lab.github.io/salmon/',
        info="is a tool for quantifying the expression of transcripts using RNA-seq data.")

        # Parse meta information. JSON win!
        self.salmon_meta = dict()
        for f in self.find_log_files('salmon/meta'):
            # Get the s_name from the parent directory
            s_name = os.path.basename( os.path.dirname(f['root']) )
            s_name = self.clean_s_name(s_name, f['root'])
            self.salmon_meta[s_name] = json.loads(f['f'])
        # Parse Fragment Length Distribution logs
        self.salmon_fld = dict()
        for f in self.find_log_files('salmon/fld'):
            # Get the s_name from the parent directory
            if os.path.basename(f['root']) == 'libParams':
                s_name = os.path.basename( os.path.dirname(f['root']) )
                s_name = self.clean_s_name(s_name, f['root'])
                parsed = OrderedDict()
                for i, v in enumerate(f['f'].split()):
                    parsed[i] = float(v)
                if len(parsed) > 0:
                    if s_name in self.salmon_fld:
                        log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                    self.add_data_source(f, s_name)
                    self.salmon_fld[s_name] = parsed

        # Filter to strip out ignored sample names
        self.salmon_meta = self.ignore_samples(self.salmon_meta)
        self.salmon_fld = self.ignore_samples(self.salmon_fld)

        if len(self.salmon_meta) == 0 and len(self.salmon_fld) == 0:
            raise UserWarning

        if len(self.salmon_meta) > 0:
            log.info("Found {} meta reports".format(len(self.salmon_meta)))
            self.write_data_file(self.salmon_meta, 'multiqc_salmon')
        if len(self.salmon_fld) > 0:
            log.info("Found {} fragment length distributions".format(len(self.salmon_fld)))

        # Add alignment rate to the general stats table
        headers = OrderedDict()
        headers['percent_mapped'] = {
            'title': '% Aligned',
            'description': '% Mapped reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlGn'
        }
        headers['num_mapped'] = {
            'title': 'M Aligned',
            'description': 'Mapped reads (millions)',
            'min': 0,
            'scale': 'PuRd',
            'modify': lambda x: float(x) / 1000000,
            'shared_key': 'read_count'
        }
        self.general_stats_addcols(self.salmon_meta, headers)

        # Fragment length distribution plot
        pconfig = {
            'smooth_points': 500,
            'id': 'salmon_plot',
            'title': 'Salmon: Fragment Length Distribution',
            'ylab': 'Fraction',
            'xlab': 'Fragment Length (bp)',
            'ymin': 0,
            'xmin': 0,
            'tt_label': '<b>{point.x:,.0f} bp</b>: {point.y:,.0f}',
        }
        self.add_section( plot = linegraph.plot(self.salmon_fld, pconfig) )



#!/usr/bin/env python

""" MultiQC submodule to parse output from Bamtools bam_stat.py
http://bamtools.sourceforge.net/#bam-stat-py """

from collections import OrderedDict
import logging
import re

from multiqc.plots import beeswarm

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find bamtools stats reports and parse their data """

    # Set up vars
    self.bamtools_stats_data = dict()
    regexes = {
        'total_reads': r"Total reads:\s*(\d+)",
        'mapped_reads': r"Mapped reads:\s*(\d+)",
        'mapped_reads_pct': r"Mapped reads:\s*\d+\s+\(([\d\.]+)%\)",
        'forward_strand': r"Forward strand:\s*(\d+)",
        'forward_strand_pct': r"Forward strand:\s*\d+\s+\(([\d\.]+)%\)",
        'reverse_strand': r"Reverse strand:\s*(\d+)",
        'reverse_strand_pct': r"Reverse strand:\s*\d+\s+\(([\d\.]+)%\)",
        'failed_qc': r"Failed QC:\s*(\d+)",
        'failed_qc_pct': r"Failed QC:\s*\d+\s+\(([\d\.]+)%\)",
        'duplicates': r"Duplicates:\s*(\d+)",
        'duplicates_pct': r"Duplicates:\s*\d+\s+\(([\d\.]+)%\)",
        'paired_end': r"Paired-end reads:\s*(\d+)",
        'paired_end_pct': r"Paired-end reads:\s*\d+\s+\(([\d\.]+)%\)",
        'proper_pairs': r"'Proper-pairs'\s*(\d+)",
        'proper_pairs_pct': r"'Proper-pairs'\s*\d+\s+\(([\d\.]+)%\)",
        'both_mapped': r"Both pairs mapped:\s*(\d+)",
        'both_mapped_pct': r"Both pairs mapped:\s*\d+\s+\(([\d\.]+)%\)",
        'read_1': r"Read 1:\s*(\d+)",
        'read_2': r"Read 2:\s*(\d+)",
        'singletons': r"Singletons:\s*(\d+)",
        'singletons_pct': r"Singletons:\s*\d+\s+\(([\d\.]+)%\)",
    }

    # Go through files and parse data using regexes
    for f in self.find_log_files('bamtools/stats'):
        d = dict()
        for k, r in regexes.items():
            r_search = re.search(r, f['f'], re.MULTILINE)
            if r_search:
                d[k] = float(r_search.group(1))

        if len(d) > 0:
            if f['s_name'] in self.bamtools_stats_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
            self.add_data_source(f, section='stats')
            self.bamtools_stats_data[f['s_name']] = d

    # Filter to strip out ignored sample names
    self.bamtools_stats_data = self.ignore_samples(self.bamtools_stats_data)

    if len(self.bamtools_stats_data) > 0:

        # Write to file
        self.write_data_file(self.bamtools_stats_data, 'multiqc_bamtools_stats')

        # Add to general stats table
        self.general_stats_headers['duplicates_pct'] = {
            'title': '% Duplicates',
            'description': '% Duplicate Reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'OrRd'
        }
        self.general_stats_headers['mapped_reads_pct'] = {
            'title': '% Mapped',
            'description': '% Mapped Reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'RdYlGn'
        }
        for s_name in self.bamtools_stats_data:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update( self.bamtools_stats_data[s_name] )

        # Make dot plot of counts
        keys = OrderedDict()
        defaults = {
            'min': 0,
            'max': 100,
            'decimalPlaces': 2,
            'suffix': '%'
        }
        num_defaults = {
            'min': 0,
            'modify': lambda x: float(x) / 1000000.0,
            'decimalPlaces': 2
        }

        keys['total_reads'] = dict(num_defaults, **{'title': 'Total reads', 'description': 'Total reads (millions)' });
        keys['mapped_reads_pct'] = dict(defaults, **{'title': 'Mapped reads' })
        keys['forward_strand_pct'] = dict(defaults, **{'title': 'Forward strand' })
        keys['reverse_strand_pct'] = dict(defaults, **{'title': 'Reverse strand' })
        keys['failed_qc_pct'] = dict(defaults, **{'title': 'Failed QC' })
        keys['duplicates_pct'] = dict(defaults, **{'title': 'Duplicates' })
        keys['paired_end_pct'] = dict(defaults, **{'title': 'Paired-end', 'description': 'Paired-end reads' })
        keys['proper_pairs_pct'] = dict(defaults, **{'title': 'Proper-pairs' })
        keys['both_mapped_pct'] = dict(defaults, **{'title': 'Both mapped', 'description': 'Both pairs mapped' })
        keys['bt_read_1'] = dict(num_defaults, **{'title': 'Read 1', 'description': 'Read 1 (millions)' });
        keys['bt_read_2'] = dict(num_defaults, **{'title': 'Read 2', 'description': 'Read 2 (millions)' });
        keys['singletons_pct'] = dict(defaults, **{'title': 'Singletons' })

        self.add_section (
            name = 'Bamtools Stats',
            anchor = 'bamtools-stats',
            plot = beeswarm.plot(self.bamtools_stats_data, keys)
        )

    # Return number of samples found
    return len(self.bamtools_stats_data)



from multiqc.modules.base_module import BaseMultiqcModule
import logging
from collections import OrderedDict
from multiqc import config
from multiqc.plots import table
import re

log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Illumina InterOp Statistics', anchor='interop',
        href="http://illumina.github.io/interop/index.html",
        info=" - a set of common routines used for reading and writing InterOp metric files.")

        log = logging.getLogger(__name__)

        # Parse data
        self.runSummary = {}
        self.indexSummary = {}
        for f in self.find_log_files('interop/summary',filehandles=True):
            parsed_data = self.parse_summary_csv(f['f'])
            if max(len(parsed_data['summary']), len(parsed_data['details'])) > 0:
                self.runSummary[f['s_name']] = parsed_data
        for f in self.find_log_files('interop/index-summary',filehandles=True):
            parsed_data = self.parse_index_summary_csv(f['f'])
            if max(len(parsed_data['summary']), len(parsed_data['details'])) > 0:
                self.indexSummary[f['s_name']] = parsed_data

        self.runSummary = self.ignore_samples(self.runSummary)
        self.indexSummary = self.ignore_samples(self.indexSummary)

        # No samples
        num_samples = max(len(self.runSummary), len(self.indexSummary))
        if num_samples == 0:
            raise UserWarning

        log.info("Found {} reports".format(num_samples))

        # Create report Sections
        if len(self.runSummary) > 0:
            self.add_section (
                name = 'Read Metrics Summary',
                anchor = 'interop-runmetrics-summary',
                description = 'Summary statistics for Total read count from each run.',
                plot = self.run_metrics_summary_table(self.runSummary)
            )
            self.add_section (
                name = 'Read Metrics per Lane',
                anchor = 'interop-runmetrics-details',
                plot = self.run_metrics_details_table(self.runSummary)
            )

        if len(self.indexSummary) > 0:
            self.add_section (
                name = 'Indexing QC Metrics summary',
                anchor = 'interop-indexmetrics-summary',
                description = 'Summary metrics about each lane',
                plot = self.index_metrics_summary_table(self.indexSummary)
            )
            self.add_section (
                name = 'Indexing QC Metrics details',
                anchor = 'interop-indexmetrics-details',
                description = ' Detail Metrics about each lane',
                plot = self.index_metrics_details_table(self.indexSummary)
            )

    def parse_summary_csv(self,f):
        metrics = {
                'summary': {},
                'details': {}
                }
        header = []
        section = None
        read = None
        for line in f:
            line = line.strip()
            # assume fixed file format
            # find summary header
            if line.startswith("Level,Yield,Projected Yield,Aligned,Error Rate,Intensity C1,%>=Q30"):
                # set section to summary
                section = "summary"
                header = line.split(",")
            elif section == "summary":
                data = line.split(",")
                # process summary
                metrics['summary'][data[0]] = {}
                for idx in range(1,len(data)):
                    try:
                        metrics['summary'][data[0]][header[idx]] = float(data[idx])
                    except ValueError:
                        metrics['summary'][data[0]][header[idx]] = data[idx]
                if line.startswith("Total"):
                    section = None
            elif line.startswith("Read") and (section is None or section == "details"):
                # set section to details
                section = "details"
                read = line
            elif line.startswith("Lane,Surface,Tiles,Density,Cluster") and section == "details":
                # get details header
                header = line.split(",")
            elif section == "details":
                if line.startswith("Extracted: "):
                    section = "finish"
                    continue
                data = line.split(",")
                # process summary
                linedata = {}
                # Check if "surface" is total (-) else skip
                if data[1] == '-':
                    metrics['details']["{} - Lane {}".format(read,data[0])] = {}
                else:
                    continue
                for idx in range(2,len(data)):
                    try:
                        if header[idx] == 'Phas/Prephas':
                            val = data[idx].split('/')
                            linedata['Phased'] = val[0]
                            linedata['Prephased'] = val[1]
                        else:
                            linedata[header[idx]] = float(data[idx])
                    except ValueError:
                        linedata[header[idx]] = re.sub(pattern='\+/-.*', repl='', string=data[idx])
                metrics['details']["Lane {} - {}".format(data[0],read)]=linedata

        return metrics

    def parse_index_summary_csv(self,f):
        metrics={'summary':{},
                 'details':{}
                 }
        header = []
        section = None
        lane = None

        summary = {}
        details = {}

        for line in f:
            line = line.strip()
            #assume fixed file format
            if line.startswith("Lane"):
                #set lane
                lane = line
                summary[lane]={}
                continue
            if line.startswith("Total Reads,PF Reads,% Read Identified (PF),CV,Min,Max"):
                header = line.split(",")
                section = "summary"
                continue
            if line.startswith("Index Number,Sample Id,Project,Index 1 (I7),Index 2 (I5),% Read Identified (PF)"):
                header = line.split(",")
                section = "details"
                continue
            if section == "summary":
                data = line.split(",")
                for idx in range(0,len(data)):
                    summary[lane][header[idx]]=data[idx]
                continue
            if section == "details":
                data = line.split(",")
                details["{} - {}".format(data[1],lane)]={}
                for idx in range(2,len(data)):
                    details["{} - {}".format(data[1],lane)][header[idx]]=data[idx]
                continue

        metrics['summary']=summary
        metrics['details']=details

        return metrics

    def run_metrics_summary_table(self, data):

        headers = OrderedDict()
        headers['Yield'] = {
            'rid': 'summary_Yield',
            'title': '{}p Yield'.format(config.base_count_prefix),
            'description': 'The number of bases sequenced ({} base pairs over all "usable cycles"'.format(config.base_count_desc),
            'scale': 'PuOr',
            'shared_key': 'base_count',
            'modify': lambda x: (x*1000000000.0) * config.base_count_multiplier, # number is already in gigabases
        }
        headers['Aligned'] = {
            'rid': 'summary_Aligned',
            'title': 'Aligned (%)',
            'description': 'The percentage of the sample that aligned to the PhiX genome',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'PiYG'
        }
        headers['Error Rate'] = {
            'title': 'Error Rate (%)',
            'description': '',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'OrRd'
        }
        headers['Intensity C1'] = {
            'rid': 'summary_Intensity_C1',
            'title': 'Intensity Cycle 1',
            'description': 'The intensity statistic at cycle 1.',
        }
        headers['%>=Q30'] = {
            'rid': 'summary_Q30',
            'title': '% >= Q30',
            'description': 'Percentage of reads with quality phred score of 30 or above',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'RdYlGn'
        }
        table_config = {
            'namespace': 'interop',
            'id': 'interop-runmetrics-summary-table',
            'table_title': 'Read metrics summary',
            'col1_header': 'Run - Read',
        }

        tdata = {}
        for s_name in data:
            for key in data[s_name]['summary']:
                tdata["{} - {}".format(s_name,key)]=data[s_name]['summary'][key]

        return table.plot(tdata, headers, table_config)

    def run_metrics_details_table(self,data):
        headers = OrderedDict()
        headers['Surface'] = {
            'title': 'Surface',
            'description': ''
        }
        headers['Tiles'] = {
            'title': 'Tiles',
            'description': 'The number of tiles per lane.',
            'hidden': True
        }
        headers['Density'] = {
            'title': 'Density',
            'description': 'The density of clusters (in thousands per mm2) detected by image analysis, +/- 1 standard deviation.',
            'hidden': True
        }
        headers['Cluster PF'] = {
            'title': 'Cluster PF (%)',
            'description': 'The percentage of clusters passing filtering, +/- 1 standard deviation.',
            'suffix': '%',
        }
        headers['Phased'] = {
            'title': 'Phased (%)',
            'description': 'The value used by RTA for the percentage of molecules in a cluster for which sequencing falls behind (phasing) or jumps ahead (prephasing) the current cycle within a read.',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'OrRd'
        }
        headers['Prephased'] = {
            'title': 'Prephased (%)',
            'description': 'The value used by RTA for the percentage of molecules in a cluster for which sequencing falls behind (phasing) or jumps ahead (prephasing) the current cycle within a read.',
            'format': '{:.,2f}',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'OrRd'
        }
        headers['Reads'] = {
            'title': '{} Reads'.format(config.read_count_prefix),
            'description': 'The number of clusters ({})'.format(config.read_count_desc),
            'shared_key': 'read_count',
            'modify': lambda x: (x*1000000.0) * config.read_count_multiplier, # number is already in millions
        }
        headers['Reads PF'] = {
            'title': '{} PF Reads'.format(config.read_count_prefix),
            'description': 'The number of passing filter clusters ({})'.format(config.read_count_desc),
            'shared_key': 'read_count',
            'modify': lambda x: (x*1000000.0) * config.read_count_multiplier, # number is already in millions
        }
        headers['Cycles Error'] = {
            'title': 'Cycles Error',
            'description': 'The number of cycles that have been error-rated using PhiX, starting at cycle 1.',
            'format': '{:.,0f}',
        }
        headers['Yield'] = {
            'title': '{}p Yield'.format(config.base_count_prefix),
            'description': 'The number of bases sequenced which passed filter ({} base pairs)'.format(config.base_count_desc),
            'scale': 'PuOr',
            'shared_key': 'base_count',
            'modify': lambda x: (x*1000000000.0) * config.base_count_multiplier, # number is already in gigabases
        }
        headers['Aligned'] = {
            'title': 'Aligned (%)',
            'description': 'The percentage that aligned to the PhiX genome.',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'PiYG'
        }
        headers['Error'] = {
            'title': 'Error Rate (%)',
            'description': 'The calculated error rate, as determined by the PhiX alignment.',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'OrRd'
        }
        headers['Error (35)'] = {
            'title': 'Error Rate 35 Cycles (%)',
            'description': 'The calculated error rate for cycles 1-35.',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'OrRd',
            'hidden': True
        }
        headers['Error (75)'] = {
            'title': 'Error Rate 75 Cycles (%)',
            'description': 'The calculated error rate for cycles 1-75.',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'OrRd',
            'hidden': True
        }
        headers['Error (100)'] = {
            'title': 'Error Rate 100 Cycles (%)',
            'description': 'The calculated error rate for cycles 1-100.',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'OrRd',
            'hidden': True
        }
        headers['Intensity C1'] = {
            'title': 'Intensity Cycle 1',
            'description': 'The intensity statistic at cycle 1.',
        }
        headers['%>=Q30'] = {
            'title': '%>=Q30',
            'description': 'The percentage of bases with a quality score of 30 or higher, respectively.',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'RdYlGn'
        }
        table_config = {
            'namespace': 'interop',
            'id': 'interop-runmetrics-detail-table',
            'table_title': 'Sequencing Lane Statistics',
            'col1_header': 'Run - Lane - Read',
        }

        tdata = {}
        for s_name in data:
            for key in data[s_name]['details']:
                tdata["{} - {}".format(s_name,key)]=data[s_name]['details'][key]

        return table.plot(tdata, headers, table_config)

    def index_metrics_summary_table(self,data):
        headers = OrderedDict()
        headers['Total Reads'] = {
            'title': '{} Reads'.format(config.read_count_prefix),
            'description': 'The total number of reads for this lane ({})'.format(config.read_count_desc),
            'modify': lambda x: float(x) * config.read_count_multiplier,
            'format': '{:,.2f}',
            'shared_key': 'read_count'
        }
        headers['PF Reads'] = {
            'title': '{} PF Reads'.format(config.read_count_prefix),
            'description': 'The total number of passing filter reads for this lane ({})'.format(config.read_count_desc),
            'modify': lambda x: float(x) * config.read_count_multiplier,
            'format': '{:,.2f}',
            'shared_key': 'read_count'
        }
        headers['% Read Identified (PF)'] = {
            'rid': 'summary_reads_identified_pf',
            'title': '% Reads Identified (PF)',
            'description': 'The total fraction of passing filter reads assigned to an index.',
            'suffix': '%',
        }
        headers['CV'] = {
            'title': 'CV',
            'description': 'The coefficient of variation for the number of counts across all indexes.',
            'format': '{:.,2f}',
        }
        headers['Min'] = {
            'title': 'Min',
            'description': 'The lowest representation for any index.'
        }
        headers['Max'] = {
            'title': 'Max',
            'description': 'The highest representation for any index.'
        }
        table_config = {
            'namespace': 'interop',
            'id': 'interop-indexmetrics-summary-table',
            'table_title': 'Index Read Statistics Summary',
            'col1_header': 'Run - Lane',
        }

        tdata = {}
        for s_name in data:
            for key in data[s_name]['summary']:
                tdata["{} - {}".format(s_name,key)]=data[s_name]['summary'][key]

        return table.plot(tdata, headers, table_config)

    def index_metrics_details_table(self,data):
        headers = OrderedDict()
        headers['% Read Identified (PF)'] = {
            'title': '% Reads Identified (PF)',
            'description': 'The number of reads (only includes Passing Filter reads) mapped to this index.',
            'suffix': '%',
        }
        headers['Index 1 (I7)'] = {
            'title': 'Index 1 (I7)',
            'description': 'The sequence for the first Index Read.'
        }
        headers['Index 2 (I5)'] = {
            'title': 'Index 2 (I5)',
            'description': 'The sequence for the second Index Read.'
        }

        table_config = {
            'namespace': 'interop',
            'id': 'interop-indexmetrics-details-table',
            'table_title': 'Index Read Statistics Details',
            'col1_header': 'Run - Sample - Lane',
        }

        tdata = {}
        for s_name in data:
            for key in data[s_name]['details']:
                tdata["{} - {}".format(s_name,key)]=data[s_name]['details'][key]

        return table.plot(tdata, headers, table_config)



#!/usr/bin/env python

""" MultiQC module to parse output from Supernova"""

from __future__ import print_function
from collections import OrderedDict
import logging
import re
import json
from multiqc.plots import table, linegraph, bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):
        super(MultiqcModule, self).__init__(name='Supernova', anchor='supernova',
        href="https://www.10xgenomics.com/",
        info="is a de novo genome assembler 10X Genomics linked-reads.")

        # Headers for the supernova Table
        self.headers = OrderedDict()
        self.headers['Asm size'] = {
                'description': 'assembly size (in megabases) ;only scaffolds >= 10 kb',
                'modify': lambda x: x / 1000000.0,
                'suffix': 'Mb',
                'scale': 'YlGn'
        }
        self.headers['% missing 10Kb'] = {
                'rid': 'pct_missing_10Kb',
                'description': '% of base assembly missing from scaffolds >= 10 kb',
                'suffix': '%',
                'scale': 'YlGn',
        }
        self.headers['# Long scaffs'] = {
                'rid': 'num_long_scaffs',
                'description': 'number of scaffolds >= 10 kb',
                'scale': 'YlGn',
                'format': '{:,.0f}',
                'hidden': True
        }
        self.headers['Scaff N50'] = {
                'description': 'N50 scaffold size (in kilobases)',
                'modify': lambda x: x / 1000.0,
                'suffix': 'Kb',
                'scale': 'RdYlGn'
        }
        self.headers['Phase N50'] = {
                'description': 'N50 phase block size (in kilobases)',
                'modify': lambda x: x / 1000.0,
                'suffix': 'Kb',
                'scale': 'RdYlGn',
                'hidden': True
        }
        self.headers['Contig N50'] = {
                'description': 'N50 contig size (in kilobases)',
                'modify': lambda x: x / 1000.0,
                'suffix': 'Kb',
                'scale': 'RdYlGn',
                'hidden': True
        }
        self.headers['Edge N50'] = {
                'description': 'N50 edge size (in kilobases)',
                'modify': lambda x: x / 1000.0,
                'suffix': 'Kb',
                'scale': 'RdYlGn',
                'hidden': True
        }
        self.headers['Mol size'] = {
                'description': 'weighted mean molecule size (in kilobases); ideal 50-100',
                'modify': lambda x: x / 1000.0,
                'suffix': 'Kb',
                'scale': 'BuGn'
        }
        self.headers['Read len'] = {
                'description': 'mean read length (in bases) after trimming; ideal 140',
                'suffix': 'b',
                'scale': 'PuBu',
                'format': '{:,.0f}',
                'hidden': True
        }
        self.headers['# Reads'] = {
                'rid': 'num_reads',
                'description': 'number of reads (in millions); ideal 800M-1200M for human',
                'modify': lambda x: x / 1000000.0,
                'suffix': 'M',
                'scale': 'PuBu',
        }
        self.headers['Raw coverage'] = {
                'description': 'raw coverage; ideal ~56',
                'suffix': 'x',
                'scale': 'PuBu',
                'hidden': True
        }
        self.headers['Coverage'] = {
                'description': 'effective read coverage; ideal ~42 for nominal 56x cov',
                'suffix': 'x',
                'scale': 'PuBu'
        }
        self.headers['% Dup'] = {
                'rid': 'pct_Dup',
                'description': 'fraction of reads that are duplicates',
                'suffix': '%',
                'scale': 'OrRd',
        }
        self.headers['% R2 Q30'] = {
                'rid': 'pct_R2_Q30',
                'description': 'fraction of Q30 bases in read 2; ideal 75-85%',
                'suffix': '%',
                'scale': 'OrRd',
        }
        self.headers['Insert size'] = {
                'description': 'median insert size (in bases); ideal 0.35-0.40 Kb',
                'suffix': 'b',
                'scale': 'OrRd',
                'format': '{:,.0f}',
                'hidden': True
        }
        self.headers['% proper'] = {
                'rid': 'pct_proper',
                'description': 'fraction of proper read pairs; ideal >= 75%',
                'suffix': '%',
                'scale': 'OrRd',
                'hidden': True
        }
        self.headers['BC usage'] = {
                'description': 'fraction of barcodes used; between 0 and 1',
                'scale': 'OrRd',
                'hidden': True
        }
        self.headers['Est size'] = {
                'description': 'estimated genome size',
                'modify': lambda x: x / 1000000.0,
                'suffix': 'Mb',
                'scale': 'YlGn',
                'hidden': True
        }
        self.headers['% repeats'] = {
                'rid': 'pct_repeats',
                'description': 'Estimated repetitive fraction (of genome)',
                'scale': 'YlGn',
                'suffix': '%',
                'hidden': True
        }
        self.headers['% AT'] = {
                'rid': 'pct_AT',
                'description': 'high AT index (of genome)',
                'scale': 'YlGn',
                'suffix': '%',
                'hidden': True
        }
        self.headers['Het dist'] = {
                'description': 'mean distance between heterozygous SNPs (in kilobases)',
                'modify': lambda x: x / 1000.0,
                'suffix': 'Kb',
                'scale': 'YlGn',
                'format': '{:,.0f}',
                'hidden': True
        }
        self.headers['p10'] = {
                'description': 'molecule count extending 10 kb on both sides',
                'scale': 'BuGn',
                'hidden': True
        }
        self.headers['% missing BC'] = {
                'rid': 'pct_missing_BC',
                'description': 'fraction of reads that are not barcoded',
                'suffix': '%',
                'scale': 'BuGn',
        }
        self.headers['Barcode N50'] = {
                'description': 'N50 reads per barcode (in bases)',
                'suffix': 'b',
                'scale': 'BuGn',
                'format': '{:,.0f}',
        }
        self.headers['% Phased'] = {
                'rid': 'pct_Phased',
                'description': 'nonduplicate and phased reads; ideal 45-50%',
                'suffix': '%',
                'scale': 'BuGn',
                'hidden': True
        }

        reports = OrderedDict()
        summaries = OrderedDict()
        molecules = OrderedDict()
        kmers = OrderedDict()
        root_summary = {}

        ### Parse the input log files
        # report.txt files
        for f in self.find_log_files('supernova/report'):
            log.debug("Found report in: {}".format(f['root']))
            sid, data = self.parse_report(f['f'])
            s_name = self.clean_s_name(sid, f['root'])
            if s_name in reports.keys():
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
            reports[s_name] = data
            self.add_data_source(f, s_name=s_name, section='supernova-table')

        # summary.json files
        for f in self.find_log_files('supernova/summary'):
            log.debug("Found summary.json in: {}".format(f['root']))
            try:
                sid, data = self.parse_summary(f['f'])
            except ValueError:
                log.debug("Error parsing JSON file in {}".format(f['root']))
                continue
            except RuntimeError:
                log.debug("Could not find sample_id in JSON file in {}".format(f['root']))
                continue

            s_name = self.clean_s_name(sid, f['root'])
            if s_name in summaries.keys():
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
            summaries[s_name] = data
            self.add_data_source(f, s_name=s_name, section='supernova-table')
            # The plot json files do not contain sample IDs, sadly. So we need to store it somewhere.
            root_summary[f['root']] = sid

        # histogram_molecules.json files
        for f in self.find_log_files('supernova/molecules'):
            log.debug("Found histogram_molecules.json in: {}".format(f['root']))
            try:
                if f['root'] in root_summary.keys():
                    data = self.parse_histogram(f['f'])
                    sid = root_summary[f['root']]
                    s_name = self.clean_s_name(sid, f['root'])
                    molecules[s_name] = data
                    self.add_data_source(f, s_name=s_name, section='supernova-molecules')
            except RuntimeError:
                log.debug("Could not parse JSON file in {}".format(f['root']))
                continue

        # histogram_kmer_count.json files
        for f in self.find_log_files('supernova/kmers'):
            log.debug("Found histogram_kmer_count.json in: {}".format(f['root']))
            try:
                if f['root'] in root_summary.keys():
                    data = self.parse_histogram(f['f'], 400)
                    sid = root_summary[f['root']]
                    s_name = self.clean_s_name(sid, f['root'])
                    kmers[s_name] = data
                    self.add_data_source(f, s_name=s_name, section='supernova-kmers')
            except RuntimeError:
                log.debug("Could not parse JSON file in {}".format(f['root']))
                continue

        # Data from summary.json supersedes data from report.txt
        for sample_id, sum_data in summaries.items():
            if sample_id in reports.keys():
                log.debug("Found summary data for sample {} which supersedes report data".format(sample_id))
                reports[sample_id] = sum_data
        # Ignore cmd-line specified samples
        reports = self.ignore_samples(reports)
        molecules = self.ignore_samples(molecules)
        kmers = self.ignore_samples(kmers)

        if len(reports) == 0:
            raise UserWarning
        else:
            log.info("Found {} reports".format(len(reports.keys())))

        ### Write the report
        self.write_data_file(reports, 'multiqc_supernova')
        config_table = {
            'id': 'supernova_table',
            'namespace': 'supernova'
        }
        self.add_section (
            name = 'Assembly statistics',
            anchor = 'supernova-table',
            description = 'Statistics gathered from the summary report(s) of Supernova. Note! ' \
                    'There are more columns available but they are hidden by default.',
            helptext = 'As a bare minimum these numbers are generated from the file report.txt, ' \
                    'found in the folder `sampleID/outs/`. If available the stats in the report ' \
                    'file will be superseded by the higher precision numbers found in the file ' \
                    '`sampleID/outs/assembly/stats/summary.json`',
            plot = table.plot(reports, self.headers, config_table)
        )

        # N50 barcharts
        n50_cats = [{'Scaff N50': {'name': 'Scaffold N50', 'color': '#66c2a5'}},
                {'Contig N50': {'name': 'Contig N50', 'color': '#fc8d62'}},
                {'Edge N50': {'name': 'Edge N50', 'color': '#8da0cb'}},
                {'Phase N50': {'name': 'Phase block N50', 'color': '#e78ac3'}}
        ]
        config_n50 = {
                'id': 'supernova_n50',
                'title': 'Supernova: N50 statistics',
                'ylab': 'Scaffold N50',
                'cpswitch': False,
                'data_labels': ['Scaffold N50', 'Contig N50', 'Edge N50', 'Phase block N50']
        }
        self.add_section (
            name = 'N50 statistics',
            anchor = 'supernova-n50',
            description = 'Assembly N50 values - the shortest sequence length at 50% of the genome when sorted by size (see [wikipedia](https://en.wikipedia.org/wiki/N50,_L50,_and_related_statistics#N50)).',
            helptext = "Note that assembly size and N50 values are computed after removing scaffolds &le; 10 kb and do not count `N`s: \n\n" \
                    "* **Scaffold N50** - N50 size of scaffolds in bases, \n" \
                    "* **Contig N50** - N50 size of contigs in bases, \n" \
                    "* **Edge N50** - N50 size of raw graph assembly edges in bases, \n" \
                    "* **Phase block N50** - N50 size of phase blocks in bases. \n\n" \
                    '[(source)](https://support.10xgenomics.com/de-novo-assembly/software/pipelines/latest/output/asm-stats)',
            plot = bargraph.plot([reports,reports,reports,reports], n50_cats, config_n50)
        )

        # Conditional sections
        if len(molecules) > 0:
            # Remove the long tail, or fail if this is a legacy empty json file
            try:
                max_x = self.trim_tail(molecules, 100000)
            except IndexError:
                log.debug('The histogram file is empty. Skipping molecule length section')
                return
            # Add molecules plot
            config_molecules = {
                'id': 'supernova_molecules',
                'title': 'Supernova: Molecule Lengths',
                'xlab': 'Inferred molecule length (bp)',
                'ylab': '# molecules',
                'smooth_points': 300,
                'smooth_points_sumcounts': True,
                'xmax': max_x
            }
            self.add_section (
                name = 'Molecule Lengths',
                anchor = 'supernova-molecules',
                description = 'Shows the inferred molecule lengths of the input 10X library.',
                helptext = 'Inferred in the `patch` step of the Supernova pipeline. It is worth ' \
                        'keeping in mind that the mean molecule length from the report is a length-weighted mean. ' \
                        'See the [source code](https://github.com/10XGenomics/supernova/search?q=lw_mean_mol_len&type=) ' \
                        'for how this value is calculated.',
                plot = linegraph.plot(molecules, config_molecules)
            )
        if len(kmers) > 0:
            # Remove the long tail, or fail if this is a legacy empty json file
            try:
                max_x = self.trim_tail(kmers, 50)
            except IndexError:
                log.debug('The histogram file is empty. Skipping kmers section')
                return
            # Add kmers plot
            config_kmers = {
                'id': 'supernova_kmers',
                'title': 'Supernova: Kmer Counts',
                'xlab': 'Filtered kmer multiplicity',
                'ylab': 'Counts',
                'smooth_points_sumcounts': False,
                'xmax': max_x
            }
            self.add_section (
                name = 'K-mer counts',
                anchor = 'supernova-kmers',
                description = 'Shows the k-mer frequencies of the input data to Supernova (after filtering).',
                helptext = 'This data is generated from k-merizing the input read data, where the sequences are ' \
                        'transformed in to the set of all possible sub-sequences of a fixed length of `K` (Supernova uses `K=48`). ' \
                        'The plot shows on the x-axis the multiplicity (i.e. how many times are they repeated) of these k-mers ' \
                        'and the y-axis the number of k-mers at this level of multiplicity. ' \
                        'A careful reading of this plot can give some insights into the levels of heterozygosity and repeats ' \
                        'in the genome that was sequenced and indications if the sequencing experiment was successful.',
                plot = linegraph.plot(kmers, config_kmers)
            )


    def parse_summary(self, content):

        stats = {
            'assembly_size': 'Asm size',
            'bases_per_read': 'Read len',
            'contig_N50': 'Contig N50',
            'dup_perc': '% Dup',
            'edge_N50': 'Edge N50',
            'effective_coverage': 'Coverage',
            'hetdist': 'Het dist',
            'lw_mean_mol_len': 'Mol size',
            'median_ins_sz': 'Insert size',
            'nreads': '# Reads',
            'phase_block_N50': 'Phase N50',
            'placed_perc': '% Phased',
            'placed_frac': '% Phased',
            'proper_pairs_perc': '% proper',
            'q30_r2_perc': '% R2 Q30',
            'rpb_N50': 'Barcode N50',
            'scaffold_N50': 'Scaff N50',
            'scaffolds_10kb_plus': '# Long scaffs',
            'valid_bc_perc': '% missing BC',
            'm10': '% missing 10Kb',
            'high_AT_index': '% AT',
            'raw_coverage': 'Raw coverage',
            'barcode_fraction': 'BC usage',
            'repfrac': 'Repeats',
            'est_genome_size': 'Est size',
            'p10': 'p10'
        }

        try:
            cdict = json.loads(content)
        except ValueError as e:
            raise e

        data = {}
        # Try to find sample_id
        sid = ''
        if 'CS_SAMPLE_ID' in cdict.keys():
            sid = cdict['CS_SAMPLE_ID'] # supernova 1.2
        elif 'sample_id' in cdict.keys():
            sid = cdict['sample_id']
        else:
            raise RuntimeError

        for key, value in cdict.items():
            if key in stats.keys():
                # Some trickery for supernova 1.1.4 compatability
                if key == 'placed_frac':
                    value = value * 100
                if key == 'valid_bc_perc':
                    value = 100 - value
                data[stats[key]] = value

        return (sid, data)


    def parse_report(self, content):
        # Some short-hands for converting the report numbers (pi is exactly three!)
        exp = {
            'K': 1000.0,
            'Kb': 1000.0,
            'kb': 1000.0,
            'Mb': 1000000.0,
            'M': 1000000.0,
            'Gb': 1000000000.0,
            'G': 1000000000.0
        }
        stats = {
                'READS': '# Reads',
                'MEAN READ LEN': 'Read len',
                'EFFECTIVE COV': 'Coverage',
                'READ TWO Q30': '% R2 Q30',
                'MEDIAN INSERT': 'Insert size',
                'PROPER PAIRS': '% proper',
                'MOLECULE LEN': 'Mol size',
                'HETDIST': 'Het dist',
                'UNBAR': '% missing BC',
                'BARCODE N50': 'Barcode N50',
                'DUPS': '% Dup',
                'PHASED': '% Phased',
                'LONG SCAFFOLDS': '# Long scaffs',
                'EDGE N50': 'Edge N50',
                'CONTIG N50': 'Contig N50',
                'PHASEBLOCK N50': 'Phase N50',
                'SCAFFOLD N50': 'Scaff N50',
                'ASSEMBLY SIZE': 'Asm size',
                'MISSING 10KB': '% missing 10Kb',
                'HIGH AT FRACTION': '% AT',
                'RAW COV': 'Raw coverage',
                'BARCODE FRACTION': 'BC usage',
                'REPETITIVE FRAC': 'Repeats',
                'EST GENOME SIZE': 'Est size',
                'P10': 'p10'
        }

        data = {}
        # Find the sample ID
        sid = ''
        sid_pat = re.compile('- \[(.+)\]')
        # [number, unit, category]
        stat_pat = re.compile('-\s+(\d+\.\d+)\s+(\S+|.)\s+= (.+) =')

        for l in content.splitlines():
            sid_m = re.match(sid_pat,l)
            stat_m = re.match(stat_pat, l)

            if sid_m is not None:
                sid = sid_m.groups()[0]
            if stat_m is None:
                continue
            stat_val = stat_m.groups()
            stat_type = stat_val[2].strip()
            # Parse the lines containing statistics
            if stat_type in stats.keys():
                try:
                    if stat_val[1] in exp.keys():
                        data[stats[stat_type]] = float(stat_val[0]) * exp[stat_val[1]]
                    else:
                        data[stats[stat_type]] = float(stat_val[0])
                except ValueError:
                    log.debug('Error in parsing sample {}, on line "{}"'.format(sid, stat_val))

        return (sid, data)


    def parse_histogram(self, content, cutoff=None):

        try:
            cdict = json.loads(content)
        except ValueError as e:
            raise e

        numbins = cdict['numbins'] + 1
        xdata = [i * cdict['binsize'] for i in range(0, numbins)]
        return {i: j for (i, j) in zip(xdata, cdict['vals'][:cutoff])}


    def trim_tail(self, plot, min_x=50, pct=0.99):
            join_plot = {}
            cuml_plot = {}
            for sample, plot_data in plot.items():
                for key, value in plot_data.items():
                    join_plot[key] = join_plot.get(key, 0) + value
            max_i = 0
            for key in join_plot.keys():
                max_i += join_plot[key]
                cuml_plot[key] = max_i
            max_x = [i for i,j in cuml_plot.items() if j <= max_i * pct][-1]
            # xlim = {, 50} at minimum
            if max_x < min_x:
                max_x = min_x
            return max_x



#!/usr/bin/env python

""" MultiQC module to parse logs from SnpEff """

from __future__ import print_function

from collections import OrderedDict
import logging
from multiqc.plots import bargraph, linegraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    """ SnpEff """

    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='SnpEff', anchor='snpeff',
            href="http://snpeff.sourceforge.net/",
            info=" is a genetic variant annotation and effect prediction "
                 "toolbox. It annotates and predicts the effects of variants "
                 "on genes (such as amino acid changes). "
        )

        self.snpeff_data = dict()
        self.snpeff_section_totals = dict()
        self.snpeff_qualities = dict()

        for f in self.find_log_files('snpeff', filehandles=True):
            self.parse_snpeff_log(f)

        # Filter to strip out ignored sample names
        self.snpeff_data = self.ignore_samples(self.snpeff_data)

        if len(self.snpeff_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.snpeff_data)))

        # Write parsed report data to a file
        self.write_data_file(self.snpeff_data, 'multiqc_snpeff')

        # General stats table
        self.general_stats()

        # Report sections
        self.add_section (
            name = 'Variants by Genomic Region',
            description = (
                """
                The stacked bar plot shows locations of detected variants in
                the genome and the number of variants for each location.
                """
            ),
            helptext = (
                """
                The upstream and downstream interval size to detect these
                genomic regions is 5000bp by default.
                """
            ),
            anchor = 'snpeff-genomic-regions',
            plot = self.count_genomic_region_plot()
        )
        self.add_section (
            name = 'Variant Effects by Impact',
            description = (
                """
                The stacked bar plot shows the putative impact of detected
                variants and the number of variants for each impact.
                """
            ),
            helptext = (
                """
                There are four levels of impacts predicted by SnpEff:

                * **High**: High impact (like stop codon)
                * **Moderate**: Middle impact (like same type of amino acid substitution)
                * **Low**: Low impact (ie silence mutation)
                * **Modifier**: No impact
                """
            ),
            anchor = 'snpeff-effects-impact',
            plot = self.effects_impact_plot()
        )
        self.add_section (
            name = 'Variants by Effect Types',
            description = (
                """
                The stacked bar plot shows the effect of variants at protein
                level and the number of variants for each effect type.
                """
            ),
            helptext = (
                """
                This plot shows the effect of variants with respect to
                the mRNA.
                """
            ),
            anchor = 'snpeff-effects',
            plot = self.effects_plot()
        )
        self.add_section (
            name = 'Variants by Functional Class',
            description = (
                """
                The stacked bar plot shows the effect of variants and
                the number of variants for each effect type.
                """
            ),
            helptext = (
                """
                This plot shows the effect of variants on the translation of
                the mRNA as protein. There are three possible cases:

                * **Silent**: The amino acid does not change.
                * **Missense**: The amino acid is different.
                * **Nonsense**: The variant generates a stop codon.
                """
            ),
            anchor = 'snpeff-functional-class',
            plot = self.effects_function_plot()
        )
        if len(self.snpeff_qualities) > 0:
            self.add_section (
                name = 'Variant Qualities',
                description = (
                    """
                    The line plot shows the quantity as function of the
                    variant quality score.
                    """
                ),
                helptext = (
                    """
                    The quality score corresponds to the QUAL column of the
                    VCF file. This score is set by the variant caller.
                    """
                ),
                anchor = 'snpeff-qualities',
                plot = self.qualities_plot()
            )


    def parse_snpeff_log(self, f):
        """ Go through log file looking for snpeff output """

        keys = {
            '# Summary table': [
                'Genome', 'Number_of_variants_before_filter',
                'Number_of_known_variants', 'Number_of_effects',
                'Genome_total_length', 'Change_rate'
            ],
            '# Effects by impact': ['HIGH', 'LOW', 'MODERATE', 'MODIFIER'],
            '# Effects by functional class': ['MISSENSE', 'NONSENSE', 'SILENT', 'Missense_Silent_ratio'],
            '# Hom/Het table': ['Het', 'Hom', 'Missing'],
            '# Ts/Tv summary': ['Transitions', 'Transversions', 'Ts_Tv_ratio'],
            '# Count by effects': 'all',
            '# Count by genomic region': 'all'
        }
        parsed_data = {}
        section = None
        for l in f['f']:
            l = l.strip()
            if l[:1] == '#':
                section = l
                self.snpeff_section_totals[section] = dict()
                continue
            s = l.split(',')

            # Quality values / counts
            if section == '# Quality':
                quals = OrderedDict()
                if l.startswith('Values'):
                    values = [int(c) for c in l.split(',')[1:] ]
                    counts = f['f'].readline()
                    counts = [int(c) for c in counts.split(',')[1:] ]
                    c = 0
                    total = sum(counts)
                    for i, v in enumerate(values):
                        if c < (total * 0.995):
                            quals[v] = counts[i]
                            c += counts[i]
                if len(quals) > 0:
                    self.snpeff_qualities[f['s_name']] = quals

            # Everything else
            elif section in keys:
                if keys[section] == 'all' or any([k in s[0].strip() for k in keys[section]]):
                    try:
                        parsed_data[ s[0].strip() ] = float(s[1].strip())
                    except ValueError:
                        parsed_data[ s[0].strip() ] = s[1].strip()
                    except IndexError:
                        pass
                    else:
                        # Parsing the number worked - add to totals
                        try:
                            self.snpeff_section_totals[section][s[0].strip()] += parsed_data[ s[0].strip() ]
                        except KeyError:
                            self.snpeff_section_totals[section][s[0].strip()] = parsed_data[ s[0].strip() ]
                    if len(s) > 2 and s[2][-1:] == '%':
                        parsed_data[ '{}_percent'.format(s[0].strip()) ] = float(s[2][:-1])

        if len(parsed_data) > 0:
            if f['s_name'] in self.snpeff_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
            self.add_data_source(f)
            self.snpeff_data[f['s_name']] = parsed_data

    def general_stats(self):
        """ Add key SnpEff stats to the general stats table """

        headers = OrderedDict()
        headers['Change_rate'] = {
            'title': 'Change rate',
            'scale': 'RdYlBu-rev',
            'min': 0,
            'format': '{:,.0f}'
        }
        headers['Ts_Tv_ratio'] = {
            'title': 'Ts/Tv',
            'description': 'Transitions / Transversions ratio',
            'format': '{:,.3f}'
        }
        headers['Number_of_variants_before_filter'] = {
            'title': 'M Variants',
            'description': 'Number of variants before filter (millions)',
            'scale': 'PuRd',
            'modify': lambda x: x / 1000000,
            'min': 0,
            'format': '{:,.2f}'
        }
        self.general_stats_addcols(self.snpeff_data, headers)


    def count_genomic_region_plot(self):
        """ Generate the SnpEff Counts by Genomic Region plot """

        # Sort the keys based on the total counts
        keys = self.snpeff_section_totals['# Count by genomic region']
        sorted_keys = sorted(keys, reverse=True, key=keys.get)

        # Make nicer label names
        pkeys = OrderedDict()
        for k in sorted_keys:
            pkeys[k] = {'name': k.replace('_', ' ').title().replace('Utr', 'UTR') }

        # Config for the plot
        pconfig = {
            'id': 'snpeff_variant_effects_region',
            'title': 'SnpEff: Counts by Genomic Region',
            'ylab': '# Reads',
            'logswitch': True
        }

        return bargraph.plot(self.snpeff_data, pkeys, pconfig)

    def effects_plot(self):
        """ Generate the SnpEff Counts by Effects plot """

        # Sort the keys based on the total counts
        keys = self.snpeff_section_totals['# Count by effects']
        sorted_keys = sorted(keys, reverse=True, key=keys.get)

        # Make nicer label names
        pkeys = OrderedDict()
        for k in sorted_keys:
            pkeys[k] = {'name': k.replace('_', ' ').title().replace('Utr', 'UTR') }

        # Config for the plot
        pconfig = {
            'id': 'snpeff_effects',
            'title': 'SnpEff: Counts by Effect Types',
            'ylab': '# Reads',
            'logswitch': False
        }
        return bargraph.plot(self.snpeff_data, pkeys, pconfig)

    def effects_impact_plot(self):
        """ Generate the SnpEff Counts by Effects Impact plot """

        # Put keys in a more logical order
        keys = [ 'MODIFIER', 'LOW', 'MODERATE', 'HIGH' ]

        # Make nicer label names
        pkeys = OrderedDict()
        for k in keys:
            pkeys[k] = {'name': k.title() }

        # Config for the plot
        pconfig = {
            'id': 'snpeff_variant_effects_impact',
            'title': 'SnpEff: Counts by Effects Impact',
            'ylab': '# Reads',
            'logswitch': True
        }

        return bargraph.plot(self.snpeff_data, pkeys, pconfig)

    def effects_function_plot(self):
        """ Generate the SnpEff Counts by Functional Class plot """

        # Cats to plot in a sensible order
        keys = [ 'SILENT', 'MISSENSE', 'NONSENSE' ]

        # Make nicer label names
        pkeys = OrderedDict()
        for k in keys:
            pkeys[k] = {'name': k.title() }

        # Config for the plot
        pconfig = {
            'id': 'snpeff_variant_effects_class',
            'title': 'SnpEff: Counts by Functional Class',
            'ylab': '# Reads',
            'logswitch': True
        }

        return bargraph.plot(self.snpeff_data, pkeys, pconfig)



    def qualities_plot(self):
        """ Generate the qualities plot """

        pconfig = {
            'smooth_points': 200,
            'id': 'snpeff_qualities',
            'title': 'SnpEff: Qualities',
            'ylab': 'Count',
            'xlab': 'Values',
            'xDecimals': False,
            'ymin': 0
        }

        return linegraph.plot(self.snpeff_qualities, pconfig)



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard ValidateSamFile """

import logging
from collections import OrderedDict

from multiqc.plots import table

# Initialise the logger
log = logging.getLogger(__name__)

# Possible warnings and descriptions
WARNING_DESCRIPTIONS = {
    'ADJACENT_INDEL_IN_CIGAR': 'CIGAR string contains an insertion (I) followed by deletion (D), or vice versa',
    'BAM_FILE_MISSING_TERMINATOR_BLOCK': 'BAM appears to be healthy, but is an older file so doesn\'t have terminator block',
    'E2_BASE_EQUALS_PRIMARY_BASE': 'Secondary base calls should not be the same as primary, unless one or the other is N',
    'INVALID_DATE_STRING': 'Date string is not ISO-8601',
    'INVALID_QUALITY_FORMAT': 'Quality encodings out of range; appear to be Solexa or Illumina when Phred expected. Avoid exception being thrown as a result of no qualities being read.',
    'MISSING_TAG_NM': 'The NM tag (nucleotide differences) is missing',
    'PAIRED_READ_NOT_MARKED_AS_FIRST_OR_SECOND': 'Pair flag set but not marked as first or second of pair',
    'RECORD_MISSING_READ_GROUP': 'A SAMRecord is found with no read group id',
}

# All possible errors and descriptions
ERROR_DESCRIPTIONS = {
    'CIGAR_MAPS_OFF_REFERENCE': 'Bases corresponding to M operator in CIGAR extend beyond reference',
    'DUPLICATE_PROGRAM_GROUP_ID': 'Same program group id appears more than once',
    'DUPLICATE_READ_GROUP_ID': 'Same read group id appears more than once',
    'EMPTY_READ': 'Indicates that a read corresponding to the first strand has a length of zero and/or lacks flow signal intensities (FZ)',
    'HEADER_RECORD_MISSING_REQUIRED_TAG': 'Header tag missing in header line',
    'HEADER_TAG_MULTIPLY_DEFINED': 'Header tag appears more than once in header line with different value',
    'INVALID_ALIGNMENT_START': 'Alignment start position is incorrect',
    'INVALID_CIGAR': 'CIGAR string error for either read or mate',
    'INVALID_FLAG_FIRST_OF_PAIR': 'First of pair flag set for unpaired read',
    'INVALID_FLAG_MATE_NEG_STRAND': 'Mate negative strand flag set for unpaired read',
    'INVALID_FLAG_MATE_UNMAPPED': 'Mate unmapped flag is incorrectly set',
    'INVALID_FLAG_NOT_PRIM_ALIGNMENT': 'Not primary alignment flag set for unmapped read',
    'INVALID_FLAG_PROPER_PAIR': 'Proper pair flag set for unpaired read',
    'INVALID_FLAG_READ_UNMAPPED': 'Mapped read flat not set for mapped read',
    'INVALID_FLAG_SECOND_OF_PAIR': 'Second of pair flag set for unpaired read',
    'INVALID_FLAG_SUPPLEMENTARY_ALIGNMENT': 'Supplementary alignment flag set for unmapped read',
    'INVALID_INDEX_FILE_POINTER': 'Invalid virtualFilePointer in index',
    'INVALID_INDEXING_BIN': 'Indexing bin set on SAMRecord does not agree with computed value',
    'INVALID_INSERT_SIZE': 'Inferred insert size is out of range',
    'INVALID_MAPPING_QUALITY': 'Mapping quality set for unmapped read or is >= 256',
    'INVALID_MATE_REF_INDEX': 'Mate reference index (MRNM) set for unpaired read',
    'INVALID_PLATFORM_VALUE': 'The read group has an invalid value set for its PL field',
    'INVALID_PREDICTED_MEDIAN_INSERT_SIZE': 'PI tag value is not numeric',
    'INVALID_REFERENCE_INDEX': 'Reference index not found in sequence dictionary',
    'INVALID_TAG_NM': 'The NM tag (nucleotide differences) is incorrect',
    'INVALID_VERSION_NUMBER': 'Does not match any of the acceptable versions',
    'MATE_CIGAR_STRING_INVALID_PRESENCE': 'A cigar string for a read whose mate is NOT mapped',
    'MATE_FIELD_MISMATCH': 'Read alignment fields do not match its mate',
    'MATE_NOT_FOUND': 'Read is marked as paired, but its pair was not found',
    'MATES_ARE_SAME_END': 'Both mates of a pair are marked either as first or second mates',
    'MISMATCH_FLAG_MATE_NEG_STRAND': 'Mate negative strand flag does not match read strand flag',
    'MISMATCH_FLAG_MATE_UNMAPPED': 'Mate unmapped flag does not match read unmapped flag of mate',
    'MISMATCH_MATE_ALIGNMENT_START': 'Mate alignment does not match alignment start of mate',
    'MISMATCH_MATE_CIGAR_STRING': 'The mate cigar tag does not match its mate\'s cigar string',
    'MISMATCH_MATE_REF_INDEX': 'Mate reference index (MRNM) does not match reference index of mate',
    'MISMATCH_READ_LENGTH_AND_E2_LENGTH': 'Lengths of secondary base calls tag values and read should match',
    'MISMATCH_READ_LENGTH_AND_QUALS_LENGTH': 'Length of sequence string and length of base quality string do not match',
    'MISMATCH_READ_LENGTH_AND_U2_LENGTH': 'Secondary base quals tag values should match read length',
    'MISSING_HEADER': 'The SAM/BAM file is missing the header',
    'MISSING_PLATFORM_VALUE': 'The read group is missing its PL (platform unit) field',
    'MISSING_READ_GROUP': 'The header is missing read group information',
    'MISSING_SEQUENCE_DICTIONARY': 'There is no sequence dictionary in the header',
    'MISSING_VERSION_NUMBER': 'Header has no version number',
    'POORLY_FORMATTED_HEADER_TAG': 'Header tag does not have colon',
    'READ_GROUP_NOT_FOUND': 'A read group ID on a SAMRecord is not found in the header',
    'RECORD_OUT_OF_ORDER': 'The record is out of order',
    'TAG_VALUE_TOO_LARGE': 'Unsigned integer tag value is deprecated in BAM. Template length',
    'TRUNCATED_FILE': 'BAM file does not have terminator block',
    'UNRECOGNIZED_HEADER_TYPE': 'Header record is not one of the standard types',
}


def _default_data_entry():
    return {'WARNING_count': 0, 'ERROR_count': 0, 'file_validation_status': 'pass'}


def parse_reports(self):
    """
        Find Picard ValidateSamFile reports and parse their data based on wether we
        think it's a VERBOSE or SUMMARY report
    """

    # Get data
    data = _parse_reports_by_type(self)

    if data:
        #  Filter to strip out ignored sample names (REQUIRED)
        data = self.ignore_samples(data)

        # Populate the general stats table
        _add_data_to_general_stats(self, data)

        # Add any found data to the report
        _add_section_to_report(self, data)

        # Write parsed data to a file
        self.write_data_file(data, 'multiqc_picard_validatesamfile')

    self.picard_ValidateSamFile_data = data  # Seems like the right thing to do
    return len(data)


def _parse_reports_by_type(self):
    """ Returns a data dictionary

        Goes through logs and parses them based on 'No errors found', VERBOSE or SUMMARY type.
    """

    data = dict()

    for file_meta in self.find_log_files('picard/sam_file_validation', filehandles=True):
        sample = file_meta['s_name']

        if sample in data:
            log.debug("Duplicate sample name found! Overwriting: {}".format(sample))

        filehandle = file_meta['f']
        first_line = filehandle.readline().rstrip()
        filehandle.seek(0)  # Rewind reading of the file

        if 'No errors found' in first_line:
            sample_data = _parse_no_error_report()
        elif first_line.startswith('ERROR') or first_line.startswith('WARNING'):
            sample_data = _parse_verbose_report(filehandle)
        else:
            sample_data = _parse_summary_report(filehandle)

        data[sample] = sample_data

    return data


def _parse_no_error_report():
    return _default_data_entry()


def _parse_verbose_report(filehandle):
    sample_data = _default_data_entry()
    errors = warnings = 0
    for line in filehandle:
        if line.startswith('WARNING:'):
            warnings += 1
        elif line.startswith('ERROR:'):
            errors += 1

    sample_data['WARNING_count'] = warnings
    sample_data['ERROR_count'] = errors

    if errors:
        sample_data['file_validation_status'] = 'fail'
    elif warnings:
        sample_data['file_validation_status'] = 'warn'

    return sample_data


def _parse_summary_report(filehandle):
    sample_data = _default_data_entry()
    for problem_type, name, count in _histogram_data(filehandle):
        sample_data[name] = count
        sample_data[problem_type+'_count'] += count

    if sample_data['ERROR_count']:
        sample_data['file_validation_status'] = 'fail'
    elif sample_data['WARNING_count']:
        sample_data['file_validation_status'] = 'warn'

    return sample_data


def _histogram_data(iterator):
    """ Yields only the row contents that contain the histogram entries """
    histogram_started = False
    header_passed = False
    for l in iterator:
        if '## HISTOGRAM' in l:
            histogram_started = True
        elif histogram_started:
            if header_passed:
                values = l.rstrip().split("\t")
                problem_type, name = values[0].split(':')
                yield problem_type, name, int(values[1])
            elif l.startswith('Error Type'):
                header_passed = True


def _add_section_to_report(self, data):
    """
        Adds found data to the report via several HTML generators
    """

    # Count samples that have errors and/or warnings
    pass_count = error_count = only_warning_count = 0
    for sample_data in data.values():
        if sample_data['file_validation_status'] == 'fail':
            error_count += 1
        elif sample_data['file_validation_status'] == 'warn':
            only_warning_count += 1
        else:
            pass_count += 1

    # Add overview note
    plot_html = []
    note_html = _generate_overview_note(
                        pass_count=pass_count,
                        only_warning_count=only_warning_count,
                        error_count=error_count,
                        total_count=len(data)
                    )
    plot_html.append(note_html)

    # Add the detailed table, but only if we have something to show
    if error_count or only_warning_count:
        table_html = _generate_detailed_table(data)
        plot_html.append(table_html)

    # Finally, add the html to the report as a section
    self.add_section(
            name='SAM/BAM File Validation',
            anchor='picard_validatesamfile',
            description=('This tool reports on the validity of a SAM or BAM '
                         'file relative to the SAM-format specification.'),
            helptext='''
            A detailed table is only shown if errors or warnings are found. Details about the errors and warnings are only shown if a `SUMMARY` report was parsed.

            For more information on the warnings, errors and possible fixes please read [this broadinstitute article](https://software.broadinstitute.org/gatk/documentation/article.php?id=7571).''',
            plot="\n".join(plot_html),
    )


def _add_data_to_general_stats(self, data):
    """
        Add data for the general stats in a Picard-module specific manner
    """
    headers = _get_general_stats_headers()

    self.general_stats_headers.update(headers)

    header_names = ('ERROR_count', 'WARNING_count', 'file_validation_status')

    general_data = dict()
    for sample in data:
        general_data[sample] = {column: data[sample][column] for column in header_names}
        if sample not in self.general_stats_data:
            self.general_stats_data[sample] = dict()
        if data[sample]['file_validation_status'] != 'pass':
            headers['file_validation_status']['hidden'] = False
        self.general_stats_data[sample].update(general_data[sample])


def _get_general_stats_headers():
    """
        Returns a header dict for the general stats collumns
    """
    headers = OrderedDict()

    headers['file_validation_status'] = {
        'title': "Validation",
        'description': 'ValidateSamFile (File Validation)'
    }

    headers['WARNING_count'] = {
        'title': "# Warnings",
        'description': 'ValidateSamFile (number of warnings)',
        'scale': 'Oranges',
        'shared_key': 'ValidateSamEntries',
        'colour': '255,237,160',
        'format': '{:.0f}',
        'hidden': True,
    }

    headers['ERROR_count'] = {
        'title': "# Errors",
        'description': 'ValidateSamFile (number of errors)',
        'scale': 'Reds',
        'shared_key': 'ValidateSamEntries',
        'colour': '252,146,114',
        'format': '{:.0f}',
        'hidden': True,
    }

    return headers


def _generate_overview_note(pass_count, only_warning_count, error_count, total_count):
    """ Generates and returns the HTML note that provides a summary of validation status. """

    note_html = ['<div class="progress">']
    pbars = [
        [ float(error_count), 'danger', 'had errors' ],
        [ float(only_warning_count), 'warning', 'had warnings' ],
        [ float(pass_count), 'success', 'passed' ]
    ]
    for b in pbars:
        if b[0]:
            note_html.append(
                '<div class="progress-bar progress-bar-{pbcol}" style="width: {pct}%" data-toggle="tooltip" title="{count} {sample} {txt}">{count}</div>'. \
                format(
                    pbcol = b[1],
                    count = int(b[0]),
                    pct = (b[0]/float(total_count))*100.0,
                    txt = b[2],
                    sample = 'samples' if b[0] > 1 else 'sample'
                )
            )
    note_html.append('</div>')

    return "\n".join(note_html)


def _generate_detailed_table(data):
    """
        Generates and retuns the HTML table that overviews the details found.
    """
    headers = _get_general_stats_headers()

    # Only add headers for errors/warnings we have found
    for problems in data.values():
        for problem in problems:
            if problem not in headers and problem in WARNING_DESCRIPTIONS:
                headers['WARNING_count']['hidden'] = False
                headers[problem] = {
                    'description': WARNING_DESCRIPTIONS[problem],
                    'namespace': 'WARNING',
                    'scale': headers['WARNING_count']['scale'],
                    'format': '{:.0f}',
                    'shared_key': 'warnings',
                    'hidden': True,  # Hide by default; to unclutter things.
                }
            if problem not in headers and problem in ERROR_DESCRIPTIONS:
                headers['ERROR_count']['hidden'] = False
                headers[problem] = {
                    'description': ERROR_DESCRIPTIONS[problem],
                    'namespace': 'ERROR',
                    'scale': headers['ERROR_count']['scale'],
                    'format': '{:.0f}',
                    'shared_key': 'errors',
                    'hidden': True,  # Hide by default; to unclutter things.
                }

    table_config = {
        'table_title': 'Picard: SAM/BAM File Validation',
    }

    return table.plot(data=data, headers=headers, pconfig=table_config)



# coding: utf-8
#!/usr/bin/env python

""" MultiQC submodule to parse output from Samtools flagstat """

import logging
import re
from collections import OrderedDict

from multiqc import config
from multiqc.plots import beeswarm

# Initialise the logger
log = logging.getLogger(__name__)

class FlagstatReportMixin():

    def parse_samtools_flagstats(self):
        """ Find Samtools flagstat logs and parse their data """

        self.samtools_flagstat = dict()
        for f in self.find_log_files('samtools/flagstat'):
            parsed_data = parse_single_report(f['f'])
            if len(parsed_data) > 0:
                if f['s_name'] in self.samtools_flagstat:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
                self.add_data_source(f, section='flagstat')
                self.samtools_flagstat[f['s_name']] = parsed_data

        # Filter to strip out ignored sample names
        self.samtools_flagstat = self.ignore_samples(self.samtools_flagstat)

        if len(self.samtools_flagstat) > 0:

            # Write parsed report data to a file (restructure first)
            self.write_data_file(self.samtools_flagstat, 'multiqc_samtools_flagstat')

            # General Stats Table
            flagstats_headers = dict()
            flagstats_headers['flagstat_total'] = {
                'title': '{} Reads'.format(config.read_count_prefix),
                'description': 'Total reads in the bam file ({})'.format(config.read_count_desc),
                'min': 0,
                'modify': lambda x: x * config.read_count_multiplier,
                'shared_key': 'read_count',
                'placement' : 100.0,
                'hidden': True

            }
            flagstats_headers['mapped_passed'] = {
                'title': '{} Reads Mapped'.format(config.read_count_prefix),
                'description': 'Reads Mapped in the bam file ({})'.format(config.read_count_desc),
                'min': 0,
                'modify': lambda x: x * config.read_count_multiplier,
                'shared_key': 'read_count',
                'placement' : 101.0
            }
            self.general_stats_addcols(self.samtools_flagstat, flagstats_headers)

            # Make dot plot of counts
            keys = OrderedDict()
            reads = {
                'min': 0,
                'modify': lambda x: float(x) * config.read_count_multiplier,
                 'suffix': '{} reads'.format(config.read_count_prefix),
                'decimalPlaces': 2,
                'shared_key': 'read_count'
            }
            keys['flagstat_total']              = dict(reads, title = 'Total Reads' )
            keys['total_passed']                = dict(reads, title = 'Total Passed QC' )
            keys['mapped_passed']               = dict(reads, title = 'Mapped' )

            if any(v.get('secondary_passed') for v in self.samtools_flagstat.values()):
                keys['secondary_passed']        = dict(reads, title = 'Secondary Alignments' )

            if any(v.get('supplementary_passed') for v in self.samtools_flagstat.values()):
                keys['supplementary_passed']    = dict(reads, title = 'Supplementary Alignments' )

            keys['duplicates_passed']           = dict(reads, title = 'Duplicates' )
            keys['paired in sequencing_passed'] = dict(reads, title = 'Paired in Sequencing' )
            keys['properly paired_passed']      = dict(reads, title = 'Properly Paired' )
            keys['with itself and mate mapped_passed'] = \
                                                  dict(reads, title = 'Self and mate mapped',
                                                              description = 'Reads with itself and mate mapped' )
            keys['singletons_passed']           = dict(reads, title = 'Singletons' )
            keys['with mate mapped to a different chr_passed'] = \
                                                  dict(reads, title = 'Mate mapped to diff chr',
                                                              description = 'Mate mapped to different chromosome' )
            keys['with mate mapped to a different chr (mapQ >= 5)_passed'] = \
                                                  dict(reads, title = 'Diff chr (mapQ >= 5)',
                                                              description = 'Mate mapped to different chromosome (mapQ >= 5)' )

            self.add_section (
                name = 'Samtools Flagstat',
                anchor = 'samtools-flagstat',
                description = 'This module parses the output from <code>samtools flagstat</code>. All numbers in millions.',
                plot = beeswarm.plot(self.samtools_flagstat, keys, {'id': 'samtools-flagstat-dp'})
            )

        # Return the number of logs that were found
        return len(self.samtools_flagstat)


# flagstat has one thing per line, documented here (search for flagstat):
# http://www.htslib.org/doc/samtools.html
flagstat_regexes = {
    'total':         r"(\d+) \+ (\d+) in total \(QC-passed reads \+ QC-failed reads\)",
    'secondary':     r"(\d+) \+ (\d+) secondary",
    'supplementary': r"(\d+) \+ (\d+) supplementary",
    'duplicates':    r"(\d+) \+ (\d+) duplicates",
    'mapped':        r"(\d+) \+ (\d+) mapped \((.+):(.+)\)",
    'paired in sequencing': r"(\d+) \+ (\d+) paired in sequencing",
    'read1':         r"(\d+) \+ (\d+) read1",
    'read2':         r"(\d+) \+ (\d+) read2",
    'properly paired': r"(\d+) \+ (\d+) properly paired \((.+):(.+)\)",
    'with itself and mate mapped': r"(\d+) \+ (\d+) with itself and mate mapped",
    'singletons':    r"(\d+) \+ (\d+) singletons \((.+):(.+)\)",
    'with mate mapped to a different chr': r"(\d+) \+ (\d+) with mate mapped to a different chr",
    'with mate mapped to a different chr (mapQ >= 5)': r"(\d+) \+ (\d+) with mate mapped to a different chr \(mapQ>=5\)",
}

def parse_single_report(file_obj):
    """
    Take a filename, parse the data assuming it's a flagstat file
    Returns a dictionary {'lineName_pass' : value, 'lineName_fail' : value}
    """
    parsed_data = {}

    re_groups = ['passed', 'failed', 'passed_pct', 'failed_pct']
    for k, r in flagstat_regexes.items():
        r_search = re.search(r, file_obj, re.MULTILINE)
        if r_search:
            for i,j in enumerate(re_groups):
                try:
                    key = "{}_{}".format(k, j)
                    val = r_search.group(i+1).strip('% ')
                    parsed_data[key] = float(val) if ('.' in val) else int(val)
                except IndexError:
                    pass # Not all regexes have percentages
                except ValueError:
                    parsed_data[key] = float('nan')
    # Work out the total read count
    try:
        parsed_data['flagstat_total'] = parsed_data['total_passed'] + parsed_data['total_failed']
    except KeyError:
        pass
    return parsed_data



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard RnaSeqMetrics """

from collections import OrderedDict
import logging
import os
import re

from multiqc.plots import linegraph, bargraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find Picard RnaSeqMetrics reports and parse their data """

    # Set up vars
    self.picard_RnaSeqMetrics_data = dict()
    self.picard_RnaSeqMetrics_histogram = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files('picard/rnaseqmetrics', filehandles=True):
        s_name = None
        in_hist = False
        for l in f['f']:
            # Catch the histogram values
            if s_name is not None and in_hist is True:
                try:
                    sections = l.split("\t")
                    pos = int(sections[0])
                    coverage = float(sections[1])
                    self.picard_RnaSeqMetrics_histogram[s_name][pos] = coverage
                except ValueError:
                    # Reset in case we have more in this log file
                    s_name = None
                    in_hist = False

            # New log starting
            if 'rnaseqmetrics' in l.lower() and 'INPUT' in l:
                s_name = None
                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip('[]'))
                    s_name = self.clean_s_name(s_name, f['root'])

            if s_name is not None:
                if 'rnaseqmetrics' in l.lower() and '## METRICS CLASS' in l:
                    if s_name in self.picard_RnaSeqMetrics_data:
                        log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
                    self.picard_RnaSeqMetrics_data[s_name] = dict()
                    self.picard_RnaSeqMetrics_histogram[s_name] = dict()
                    self.add_data_source(f, s_name, section='RnaSeqMetrics')
                    keys = f['f'].readline().strip("\n").split("\t")
                    vals = f['f'].readline().strip("\n").split("\t")
                    for i, k in enumerate(keys):
                        # Multiply percentages by 100
                        if k.startswith('PCT_'):
                            try:
                                vals[i] = float(vals[i]) * 100.0
                            except (ValueError, IndexError):
                                pass
                        # Save the key:value pairs
                        try:
                            self.picard_RnaSeqMetrics_data[s_name][k] = float(vals[i])
                        except ValueError:
                            self.picard_RnaSeqMetrics_data[s_name][k] = vals[i]
                        except IndexError:
                            pass # missing data
                    # Calculate some extra numbers
                    if 'PF_BASES' in keys and 'PF_ALIGNED_BASES' in keys:
                        self.picard_RnaSeqMetrics_data[s_name]['PF_NOT_ALIGNED_BASES'] = \
                            self.picard_RnaSeqMetrics_data[s_name]['PF_BASES'] - self.picard_RnaSeqMetrics_data[s_name]['PF_ALIGNED_BASES']

            if s_name is not None and 'normalized_position	All_Reads.normalized_coverage' in l:
                self.picard_RnaSeqMetrics_histogram[s_name] = dict()
                in_hist = True

        for key in list(self.picard_RnaSeqMetrics_data.keys()):
            if len(self.picard_RnaSeqMetrics_data[key]) == 0:
                self.picard_RnaSeqMetrics_data.pop(key, None)
        for s_name in list(self.picard_RnaSeqMetrics_histogram.keys()):
            if len(self.picard_RnaSeqMetrics_histogram[s_name]) == 0:
                self.picard_RnaSeqMetrics_histogram.pop(s_name, None)
                log.debug("Ignoring '{}' histogram as no data parsed".format(s_name))

    # Filter to strip out ignored sample names
    self.picard_RnaSeqMetrics_data = self.ignore_samples(self.picard_RnaSeqMetrics_data)

    if len(self.picard_RnaSeqMetrics_data) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_RnaSeqMetrics_data, 'multiqc_picard_RnaSeqMetrics')

        # Add to general stats table
        GenStatsHeaders = OrderedDict()
        GenStatsHeaders['PCT_RIBOSOMAL_BASES'] = {
            'title': '% rRNA',
            'description': 'Percent of aligned bases overlapping ribosomal RNA regions',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'Reds',
        }
        GenStatsHeaders['PCT_MRNA_BASES'] = {
            'title': '% mRNA',
            'description': 'Percent of aligned bases overlapping UTRs and coding regions of mRNA transcripts',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'Greens',
        }
        self.general_stats_addcols(self.picard_RnaSeqMetrics_data, GenStatsHeaders)

        # Bar plot of bases assignment
        bg_cats = OrderedDict()
        bg_cats['CODING_BASES'] = { 'name': 'Coding' }
        bg_cats['UTR_BASES'] = { 'name': 'UTR' }
        bg_cats['INTRONIC_BASES'] = { 'name': 'Intronic' }
        bg_cats['INTERGENIC_BASES'] = { 'name': 'Intergenic' }
        bg_cats['RIBOSOMAL_BASES'] = { 'name': 'Ribosomal' }
        bg_cats['PF_NOT_ALIGNED_BASES'] = { 'name': 'PF not aligned' }

        # Warn user if any samples are missing 'RIBOSOMAL_BASES' data; ie picard was run without an rRNA interval file.
        warn_rrna = ''
        rrna_missing = []
        for s_name, metrics in self.picard_RnaSeqMetrics_data.items():
            if metrics['RIBOSOMAL_BASES'] == '':
                rrna_missing.append(s_name)
        if len(rrna_missing):
            if len(rrna_missing) < 5:
                missing_samples = 'for samples <code>{}</code>'.format('</code>, <code>'.join(rrna_missing))
            else:
                missing_samples = '<strong>{} samples</strong>'.format(len(rrna_missing))
            warn_rrna = '''
            <div class="alert alert-warning">
              <span class="glyphicon glyphicon-warning-sign"></span>
              Picard was run without an rRNA annotation file {}, therefore the ribosomal assignment is not available. To correct, rerun with the <code>RIBOSOMAL_INTERVALS</code> parameter, as documented <a href="https://broadinstitute.github.io/picard/command-line-overview.html#CollectRnaSeqMetrics" target="_blank">here</a>.
            </div>
            '''.format(missing_samples)

        pconfig = {
            'id': 'picard_rnaseqmetrics_assignment_plot',
            'title': 'Picard: RnaSeqMetrics Read Assignments',
            'ylab': 'Number of reads'
        }
        self.add_section (
            name = 'RnaSeqMetrics Assignment',
            anchor = 'picard-rna-assignment',
            description = 'Number of bases in primary alignments that align to regions in the reference genome.' + warn_rrna,
            plot = bargraph.plot(self.picard_RnaSeqMetrics_data, bg_cats, pconfig)
        )

        # Bar plot of strand mapping
        bg_cats = OrderedDict()
        bg_cats['CORRECT_STRAND_READS'] = { 'name': 'Correct' }
        bg_cats['INCORRECT_STRAND_READS'] = { 'name': 'Incorrect', 'color': '#8e123c' }

        pdata = dict()
        for s_name, d in self.picard_RnaSeqMetrics_data.items():
            if d['CORRECT_STRAND_READS'] > 0 and d['INCORRECT_STRAND_READS'] > 0:
                pdata[s_name] = d
        if len(pdata) > 0:
            pconfig = {
                'id': 'picard_rnaseqmetrics_strand_plot',
                'title': 'Picard: RnaSeqMetrics Strand Mapping',
                'ylab': 'Number of reads',
                'hide_zero_cats': False
            }
            self.add_section (
                name = 'RnaSeqMetrics Strand Mapping',
                anchor = 'picard-rna-strand',
                description = 'Number of aligned reads that map to the correct strand.',
                plot = bargraph.plot(self.picard_RnaSeqMetrics_data, bg_cats, pconfig)
            )

        # Section with histogram plot
        if len(self.picard_RnaSeqMetrics_histogram) > 0:
            # Plot the data and add section
            pconfig = {
                'smooth_points': 500,
                'smooth_points_sumcounts': [True, False],
                'id': 'picard_rna_coverage',
                'title': 'Picard: Normalized Gene Coverage',
                'ylab': 'Coverage',
                'xlab': 'Percent through gene',
                'xDecimals': False,
                'tt_label': '<b>{point.x}%</b>: {point.y:.0f}',
                'ymin': 0,
            }
            self.add_section (
                name = 'Gene Coverage',
                anchor = 'picard-rna-coverage',
                plot = linegraph.plot(self.picard_RnaSeqMetrics_histogram, pconfig)
            )


    # Return the number of detected samples to the parent module
    return len(self.picard_RnaSeqMetrics_data)



#!/usr/bin/env python

""" MultiQC module to parse Stacks 2 denovo output"""

from __future__ import print_function
from collections import OrderedDict
import logging
import re
import os
from multiqc.plots import table, linegraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):
        super(MultiqcModule, self).__init__(name='Stacks', anchor='stacks',
        href="http://catchenlab.life.illinois.edu/stacks/",
        info="A software for analyzing restriction enzyme-based data (e.g. RAD-seq).")

        self.gsheaders = OrderedDict()
        self.gsheaders['n_loci'] = {
                'title': '# loci',
                'description': 'Number of loci built',
                'format': '{:,.i}',
                'scale': 'RdYlGn'
        }
        self.gsheaders['n_used_fw_reads'] = {
                'title': 'K reads used',
                'modify': lambda x: float(x) / 1000.0,
                'description': 'Number of thousand reads used',
                'scale': 'BuGn'
        }
        self.gsheaders['mean_cov'] = {
                'title': 'cov',
                'suffix': 'X',
                'description': 'Mean sequence coverage at locus',
                'scale': 'BuPu',
        }
        self.gsheaders['mean_cov_ns'] = {
                'title': 'weighted cov',
                'suffix': 'X',
                'description': 'The coverage at each locus is weighted by the number of samples present at that locus (i.e. coverage at shared loci counts more)',
                'scale': 'YlGn',
        }

        self.sheaders = OrderedDict()
        self.sheaders['# Pop ID'] = {
                'title': 'PopID',
                'description': 'Population ID as defined in the Population Map file.',
                'scale': False,
                'format': '{:,.s}'
        }
        self.sheaders['Private'] = {
                'title': 'Private',
                'description': 'Number of private alleles in this population.',
                'scale': 'PuBu',
                'hidden': True
        }
        self.sheaders['Num_Indv'] = {
                'title': '# Indv',
                'description': 'Mean number of individuals per locus in this population.',
                'scale': 'YlGn'
        }
        self.sheaders['P'] = {
                'title': 'P',
                'description': 'Mean frequency of the most frequent allele at each locus in this population.',
                'scale': 'PuBu',
                'min': 0,
                'max': 1
        }
        self.sheaders['Obs_Het'] = {
                'title': 'Obs Het',
                'description': 'Mean observed heterozygosity in this population.',
                'scale': 'YlGn',
                'min': 0,
                'max': 1,
        }
        self.sheaders['Obs_Hom'] = {
                'title': 'Obs Hom',
                'description': 'Mean observed homozygosity in this population.',
                'scale': 'PuBu',
                'min': 0,
                'max': 1,
                'hidden': True
        }
        self.sheaders['Exp_Hom'] = {
                'title': 'Exp_Hom',
                'description': 'Mean expected homozygosity in this population.',
                'scale': 'YlGn',
                'min': 0,
                'max': 1,
                'hidden': True
        }
        self.sheaders['Exp_Het'] = {
                'title': 'Exp Het',
                'description': 'Mean expected heterozygosity in this population.',
                'scale': 'PuBu',
                'min': 0,
                'max': 1
        }
        self.sheaders['Pi'] = {
                'title': 'Pi',
                'description': 'Mean value of &#960; in this population.',
                'scale': 'YlGn',
                'min': 0,
                'max': 1
        }
        self.sheaders['Fis'] = {
                'title': 'Fis',
                'description': 'Mean measure of Fis in this population.',
                'scale': 'PuOr',
                'min': -1,
                'max': 1
        }

        num_files = 0
        # Parse gstacks data
        self.cov_data = OrderedDict()
        for f in self.find_log_files('stacks/gstacks'):
            run_name = os.path.dirname(f['root'])
            s_name = self.clean_s_name(os.path.basename(f['root']), run_name)
            try:
                self.cov_data.update(self.parse_gstacks(f['f'], s_name))
                num_files += 1
            except:
                log.error('Could not parse gstacks.distribs file in {}'.format(f['s_name']))

        # Parse populations data
        self.distribs_loci = OrderedDict()
        self.distribs_snps = OrderedDict()
        for f in self.find_log_files('stacks/populations'):
            run_name = os.path.dirname(f['root'])
            s_name = self.clean_s_name(os.path.basename(f['root']), run_name)
            i,j = self.parse_populations(f['f'], s_name)
            try:
                self.distribs_loci.update(i); self.distribs_snps.update(j)
                num_files += 1
            except:
                log.error('Could not parse population.log.distribs file in {}'.format(f['s_name']))

        # Parse sumstats file
        self.sumstats_data = OrderedDict()
        for f in self.find_log_files('stacks/sumstats'):
            run_name = os.path.dirname(f['root'])
            s_name = self.clean_s_name(os.path.basename(f['root']), run_name)
            try:
                self.sumstats_data.update(self.parse_sumstats(f['f'], s_name))
                num_files += 1
            except:
                log.error('Could not parse populations.sumstats_summary file in {}'.format(f['s_name']))


        # Ignore samples
        self.cov_data = self.ignore_samples(self.cov_data)
        self.distribs_loci = self.ignore_samples(self.distribs_loci)
        self.distribs_snps = self.ignore_samples(self.distribs_snps)
        self.sumstats_data = self.ignore_samples(self.sumstats_data)

        if len(self.cov_data) == 0 and len(self.sumstats_data) == 0 and len(self.distribs_loci) == 0:
            raise UserWarning
        log.info("Found {} reports".format(num_files))

        # Write parsed report data to a file
        self.write_data_file(self.cov_data, 'multiqc_stacks_cov')
        self.write_data_file(self.sumstats_data, 'multiqc_stacks_sumstats')

        ### Write the sample table
        config_table = {
            'id': 'gstacks_table',
            'namespace': 'stacks'
        }
        self.add_section (
            name = 'Sample statistics',
            anchor = 'stacks-gstacks',
            description = 'The sample specific statistics for Stacks',
            helptext = '''**Note!** The sample names have the following scheme `<run folder name> | <input fastq file prefix>`.
                        This data is obtained from the gstacks program run after builing sample and catalog loci merge
                        paired-ends and call variants.
                        These numbers are obtained from the `gstacks.log.distribs` file''',
            plot = table.plot(self.cov_data, self.gsheaders, config_table)
        )
        # Write population sumstats table
        config_table = {
            'id': 'sumstats_table',
            'namespace': 'stacks'
        }
        self.add_section (
            name = 'Population summary statistics',
            anchor = 'stacks-sumstats',
            description = 'Population statistics as calculated from variant sites found in this run',
            helptext = '''**Note!** The sample names have the following scheme `<run folder name> | <population ID>`,
                        where the population ID is defined in the input population map file.
                        This information is obtained from the Stacks program `population` and the file populations.sumstats_summary.tsv
                        ''',
            plot = table.plot(self.sumstats_data, self.sheaders, config_table)
        )
        config_distribs = {
            'id': 'distribs_plot',
            'title': 'Stacks: Population plots',
            'namespace': 'stacks',
            'tt_label': '{point.y} loci, {point.x} samples/SNPs',
            'ylab': '# loci',
            'data_labels': [
                {'name': 'Samples per loci', 'ylab': '# loci', 'xlab': '# samples'},
                {'name': 'SNPs per loci', 'ylab': '# loci', 'xlab': '# SNPs'}
            ]
        }
        self.add_section (
            name = 'Population plots',
            anchor = 'stacks-distribs',
            description = 'Plots showing, 1) the number of loci shared by number of samples and 2) the number of SNPs per sample',
            helptext = '''The distributions are obtained from the Stacks program `populations` and it's output file `populations.log.distribs`.
            These numbers are Stacks' post-filtering.''',
            plot = linegraph.plot([self.distribs_loci, self.distribs_snps], config_distribs)

        )

    def parse_gstacks(self, file_contents, s_name):

        headers = None
        content = None
        out_dict = OrderedDict()

        for l in file_contents.splitlines():
            if l.startswith("sample	n_loci	n_used_fw_reads"):
                headers = ["n_loci", "n_used_fw_reads", "mean_cov","mean_cov_ns"]
            elif l.startswith("END effective_coverages_per_sample"):
                break
            elif headers is not None:
                cdict = OrderedDict().fromkeys(headers)
                content = l.split("\t")
                for i in range(1, len(content)):
                    cdict[list(cdict.keys())[i-1]] = content[i]
                if s_name is not None:
                    out_dict[s_name+" | "+content[0]] = cdict
                else:
                    out_dict[content[0]] = cdict

        return out_dict

    def parse_sumstats(self, file_contents, s_name):

        out_dict = dict()
        # ["# Pop ID","Private","Num_Indv","P","Obs_Het","Obs_Hom","Exp_Het","Exp_Hom","Pi","Fis"]
        fields = [0,1,2,5,8,11,14,17,20,23]
        fl = file_contents.splitlines()
        var_lines = fl[fl.index("# Variant positions")+1:fl.index("# All positions (variant and fixed)")]
        headers = None
        for l in var_lines:
            if l.startswith("# Pop ID	Private	Num_Indv"):
                headers = l.split("\t")
            elif headers is not None:
                cdict = OrderedDict()
                content = l.split("\t")
                for i in fields:
                    cdict[headers[i]] = content[i]
                out_dict[s_name + " | "+ content[0]] = cdict

        return out_dict


    def parse_populations(self, file_contents, s_name):

        loci_dict = dict()
        snps_dict = dict()
        pat = re.compile("BEGIN (.*)\n\n{0,1}#.*\n.+\n((?:.+\n)+)END", re.MULTILINE)
        for match in pat.finditer(file_contents):
            title, table = match.groups()
            title = title.strip()
            if title == "missing_samples_per_loc_postfilters":
                if s_name is not None:
                    title = s_name
                loci_dict[title]=dict()
                for l in table.splitlines():
                    row = l.split()
                    loci_dict[title][int(row[0])] = int(row[1])
            elif title == "snps_per_loc_postfilters":
                if s_name is not None:
                    title = s_name
                snps_dict[title]=dict()
                for l in table.splitlines():
                    row = l.split()
                    snps_dict[title][int(row[0])] = int(row[1])

        return loci_dict, snps_dict



#!/usr/bin/env python

""" MultiQC module to parse output from ClipAndMerge """

from __future__ import print_function
from collections import OrderedDict
import logging
import os
import re

from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ ClipAndMerge module """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='ClipAndMerge', anchor='clipandmerge',
        href="http://www.github.com/apeltzer/ClipAndMerge",
        info="is a tool for adapter clipping and read merging for ancient DNA data.")

        # Find and load any ClipAndMerge reports
        self.clipandmerge_data = dict()
        for f in self.find_log_files('clipandmerge'):
            self.parse_clipandmerge_log(f)

        # Filter to strip out ignored sample names
        self.clipandmerge_data = self.ignore_samples(self.clipandmerge_data)

        if len(self.clipandmerge_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.clipandmerge_data)))

        # Write parsed report data to a file
        self.write_data_file(self.clipandmerge_data, 'multiqc_clipandmerge')

        # Basic Stats Table
        self.clipandmerge_general_stats_table()

        # Alignment Rate Plot
        self.add_section(plot = self.clipandmerge_alignment_plot())


    def parse_clipandmerge_log(self, f):
        regexes = {
            'usable_reads': r"Number of usable reads in the output file\(s\):\s+(\d+)",
            'merged_reads': r"Number of usable merged reads:\s+(\d+)",
            'percentage': r"Percentage of usable merged reads:\s+(\d+\.\d+)",
            'usable_not_merged_forward': r"Number of usable not merged forward reads:\s+(\d+)",
            'usable_not_merged_reverse': r"Number of usable not merged reverse reads:\s+(\d+)",
            'usable_forward_no_pairing_reverse': r"Number of usable forward reads with no pairing reverse read:\s+(\d+)",
            'usable_reverse_no_pairing_forward': r"Number of usable reverse reads with no pairing forward read:\s+(\d+)",
            'identifier' : r"SampleID:\s+(\S+)"
        }

        parsed_data = dict()
        for k, r in regexes.items():
            r_search = re.search(r, f['f'], re.MULTILINE)
            if r_search:
                try:
                    parsed_data[k] = float(r_search.group(1))
                except ValueError:
                    parsed_data[k] = r_search.group(1)

        if len(parsed_data) > 0:
            s_name = self.clean_s_name(os.path.basename(f['root']), f['root'])
            if 'identifier' in parsed_data:
                s_name = self.clean_s_name(parsed_data['identifier'], f['root'])
            self.clipandmerge_data[s_name] = parsed_data

    def clipandmerge_general_stats_table(self):
        """ Take the parsed stats from the ClipAndMerge report and add it to the
        basic stats table at the top of the report """

        headers = OrderedDict()
        headers['percentage'] = {
            'title': '% Merged',
            'description': 'Percentage of reads merged',
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'Greens',
            'format': '{:,.2f}',
        }
        self.general_stats_addcols(self.clipandmerge_data, headers)

    def clipandmerge_alignment_plot (self):
        """ Make the HighCharts HTML to plot the duplication rates """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['merged_reads'] = { 'name': 'Merged Reads' }
        keys['usable_not_merged_forward'] = { 'name': 'Usable, not merged (forward)'}
        keys['usable_not_merged_reverse'] = { 'name': 'Usable, not merged (reverse)'}
        keys['usable_forward_no_pairing_reverse'] = { 'name': 'Usable forward-only'}
        keys['usable_reverse_no_pairing_forward'] = { 'name': 'Usable reverse-only'}

        # Config for the plot
        config = {
            'id': 'clipandmerge_rates',
            'title': 'ClipAndMerge: Read merging results',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads',
            'hide_zero_cats': False
        }

        return bargraph.plot(self.clipandmerge_data, keys, config)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" MultiQC module to parse output files from miRTrace """

from __future__ import print_function
from collections import OrderedDict
import logging

import json
from multiqc.plots import bargraph, linegraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name = 'miRTrace',
            anchor = 'mirtrace',
            href = 'https://github.com/friedlanderlab/mirtrace',
            info = "is a quality control software for small RNA sequencing data developed by Friedländer lab (KTH, Sweden)."
        )

        # Find and load miRTrace summary statistics table
        self.summary_data = dict()
        for f in self.find_log_files('mirtrace/summary'):
            self.parse_summary(f)

        # Find and load miRTrace read length table
        self.length_data = dict()
        for f in self.find_log_files('mirtrace/length'):
            self.parse_length(f)

        # Find and load miRTrace contamination statistics summary_table
        self.contamination_data = dict()
        for f in self.find_log_files('mirtrace/contaminationbasic'):
            self.parse_contamination(f)

        # Find and load miRTrace miRNA complexity table
        self.complexity_data = dict()
        for f in self.find_log_files('mirtrace/mirnacomplexity'):
            self.parse_complexity(f)

        # Filter to strip out ignored sample names
        self.summary_data         =   self.ignore_samples(self.summary_data)
        self.length_data          =   self.ignore_samples(self.length_data)
        self.contamination_data   =   self.ignore_samples(self.contamination_data)
        self.complexity_data      =   self.ignore_samples(self.complexity_data)

        # Warning when no files are found
        if max(len(self.summary_data), len(self.length_data), len(self.contamination_data), len(self.complexity_data)) == 0:
            raise UserWarning

        # Write parsed data to a file
        self.write_data_file(self.summary_data, 'multiqc_mirtrace_summary')
        self.write_data_file(self.length_data, 'multiqc_mirtrace_length')
        self.write_data_file(self.contamination_data, 'multiqc_mirtrace_contamination')
        self.write_data_file(self.complexity_data, 'multiqc_mirtrace_complexity')

        # Report sections
        if len(self.summary_data) > 0:
            self.add_section (
                name = 'QC Plot',
                anchor = 'mirtrace_qc',
                plot = self.mirtrace_qc_plot()
            )
            self.add_section (
                name = 'RNA Categories',
                anchor = 'mirtrace_rna_categories',
                plot = self.mirtrace_rna_categories()
            )

        if len(self.length_data) > 0:
            self.add_section (
                name = 'Read Length Distribution',
                anchor = 'mirtrace_length',
                plot = self.mirtrace_length_plot()
            )

        if len(self.contamination_data) > 0:
            self.add_section (
                name = 'Contamination Check',
                anchor = 'mirtrace_contamination_check',
                plot = self.mirtrace_contamination_check()
            )

        if len(self.complexity_data) > 0:
            self.add_section (
                name = 'miRNA Complexity',
                anchor = 'mirtrace_complexity',
                plot = self.mirtrace_complexity_plot()
            )

    # Parse a miRTrace results.json file
    def parse_summary(self, f):

        try:
            cdict = json.loads(f['f'])
        except ValueError as e:
            raise e

        if 'results' in cdict.keys():
            for record in cdict['results']:
                s_name = self.clean_s_name(record['verbosename'], f['root'])
                parsed_data = {}
                parsed_data['filename'] = record['filename']
                parsed_data['reads_total'] = record['stats']['allSeqsCount']
                parsed_data['adapter_removed_length_ok'] = record['stats']['statsQC'][4]
                parsed_data['adapter_not_detected'] = record['stats']['statsQC'][3]
                parsed_data['length_shorter_than_18'] = record['stats']['statsQC'][2]
                parsed_data['low_complexity'] = record['stats']['statsQC'][1]
                parsed_data['low_phred'] = record['stats']['statsQC'][0]
                parsed_data['reads_mirna'] = record['stats']['statsRNAType'][0]
                parsed_data['reads_rrna'] = record['stats']['statsRNAType'][1]
                parsed_data['reads_trna'] = record['stats']['statsRNAType'][2]
                parsed_data['reads_artifact'] = record['stats']['statsRNAType'][3]
                parsed_data['reads_unknown'] = record['stats']['statsRNAType'][4]
                if s_name in self.summary_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                self.add_data_source(f, s_name)
                self.summary_data[s_name] = parsed_data
        else:
            log.debug('No valid data {} in miRTrace summary'.format(f['fn']))
            return None


    # Parse a miRTrace mirtrace-stats-length.tsv file
    def parse_length(self, f):
        header =[]
        body = {}
        lines = f['f'].splitlines()
        for l in lines:
            s = l.split("\t")
            if len(header) == 0:
                if s[0] != 'LENGTH':
                    log.debug('No valid data {} for read length distribution'.format(f['fn']))
                    return None
                header = s[1:]
            else:
                body[s[0]]=s[1:len(s)]

        for record in header[0:len(header)]:
            s_name = self.clean_s_name(record, f['root'])
            parsed_data = {}
            idx = header[0:len(header)].index(record)
            for length in body:
                parsed_data[length] = int(body[length][idx])
            if s_name in self.length_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
            self.add_data_source(f, s_name)
            self.length_data[s_name] = parsed_data


    # Parse a miRTrace mirtrace-stats-contamination_basic.tsv file
    def parse_contamination(self, f):
        header =[]
        body = {}
        lines = f['f'].splitlines()
        for l in lines:
            s = l.split("\t")
            if len(header) == 0:
                if s[0] != 'CLADE':
                    log.debug('No valid data {} for contamination check'.format(f['fn']))
                    return None
                header = s[1:]
            else:
                body[s[0]]=s[1:len(s)]

        for record in header[0:len(header)]:
            s_name = self.clean_s_name(record, f['root'])
            parsed_data = {}
            idx = header[0:len(header)].index(record)
            for clade in body:
                parsed_data[clade] = int(body[clade][idx])
            if s_name in self.contamination_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
            self.add_data_source(f, s_name)
            self.contamination_data[s_name] = parsed_data


    # Parse a miRTrace mirtrace-stats-mirna-complexity.tsv file
    def parse_complexity(self, f):
        header =[]
        body = {}
        lines = f['f'].splitlines()
        for l in lines:
            s = l.split("\t")
            if len(header) == 0:
                if s[0] != 'DISTINCT_MIRNA_HAIRPINS_ACCUMULATED_COUNT':
                    log.debug('No valid data {} for miRNA complexity'.format(f['fn']))
                    return None
                header = s[1:]
            else:
                body[s[0]]=s[1:len(s)]

        for record in header[0:len(header)]:
            s_name = self.clean_s_name(record, f['root'])
            parsed_data = {}
            idx = header[0:len(header)].index(record)
            for depth in body:
                parsed_data[depth] = int(body[depth][idx]) if body[depth][idx] else 0
            if s_name in self.complexity_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
            self.add_data_source(f, s_name)
            self.complexity_data[s_name] = parsed_data


    # miRTrace QC Plot
    def mirtrace_qc_plot(self):
        """ Generate the miRTrace QC Plot"""

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['adapter_removed_length_ok']   =   { 'color': '#006837', 'name': 'Reads ≥ 18 nt after adapter removal' }
        keys['adapter_not_detected']        =   { 'color': '#66bd63', 'name': 'Reads without adapter' }
        keys['length_shorter_than_18']      =   { 'color': '#fdae61', 'name': 'Reads < 18 nt after adapter removal' }
        keys['low_complexity']              =   { 'color': '#d73027', 'name': 'Reads with low complexity' }
        keys['low_phred']                   =   { 'color': '#a50026', 'name': 'Reads with low PHRED score' }

        # Config for the plot
        config = {
            'id': 'mirtrace_qc_plot',
            'title': 'miRTrace: QC Plot',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        return bargraph.plot(self.summary_data, keys, config)


    # miRTrace Read Length Distribution
    def mirtrace_length_plot(self):
        """ Generate the miRTrace Read Length Distribution"""

        data = dict()
        for s_name in self.length_data:
            try:
                data[s_name] = {int(d): int(self.length_data[s_name][d]) for d in self.length_data[s_name]}
            except KeyError:
                pass
        if len(data) == 0:
            log.debug('No valid data for read length distribution')
            return None

        config = {
            'id': 'mirtrace_length_plot',
            'title': 'miRTrace: Read Length Distribution',
            'ylab': 'Read Count',
            'xlab': 'Read Lenth (bp)',
            'ymin': 0,
            'xmin': 0,
            'xDecimals': False,
            'tt_label': '<b>Read Length (bp) {point.x}</b>: {point.y} Read Count',
            'xPlotBands': [
                {'from': 40, 'to': 50, 'color': '#ffebd1'},
                {'from': 26, 'to': 40, 'color': '#e2f5ff'},
                {'from': 18, 'to': 26, 'color': '#e5fce0'},
                {'from': 0, 'to': 18, 'color': '#ffffe2'},
            ]
        }

        return linegraph.plot(data, config)


    # miRTrace RNA Categories
    def mirtrace_rna_categories(self):
        """ Generate the miRTrace RNA Categories"""

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['reads_mirna']      =   { 'color': '#33a02c', 'name': 'miRNA' }
        keys['reads_rrna']       =   { 'color': '#ff7f00', 'name': 'rRNA' }
        keys['reads_trna']       =   { 'color': '#1f78b4', 'name': 'tRNA' }
        keys['reads_artifact']   =   { 'color': '#fb9a99', 'name': 'Artifact' }
        keys['reads_unknown']    =   { 'color': '#d9d9d9', 'name': 'Unknown' }

        # Config for the plot
        config = {
            'id': 'mirtrace_rna_categories_plot',
            'title': 'miRTrace: RNA Categories',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        return bargraph.plot(self.summary_data, keys, config)


    # miRTrace Contamination Check
    def mirtrace_contamination_check(self):
        """ Generate the miRTrace Contamination Check"""

        # A library of 24 colors. Should be enough for this plot
        color_lib = ['rgb(166,206,227)', 'rgb(31,120,180)', 'rgb(178,223,138)', 'rgb(51,160,44)', 'rgb(251,154,153)', 'rgb(227,26,28)', 'rgb(253,191,111)', 'rgb(255,127,0)', 'rgb(202,178,214)', 'rgb(106,61,154)', 'rgb(255,255,153)', 'rgb(177,89,40)', 'rgb(141,211,199)', 'rgb(255,255,179)', 'rgb(190,186,218)', 'rgb(251,128,114)', 'rgb(128,177,211)', 'rgb(253,180,98)', 'rgb(179,222,105)', 'rgb(252,205,229)', 'rgb(217,217,217)', 'rgb(188,128,189)', 'rgb(204,235,197)', 'rgb(255,237,111)']

        idx = 0

        # Specify the order of the different possible categories
        keys = OrderedDict()
        for clade in self.contamination_data[list(self.contamination_data.keys())[0]]:
            keys[clade] = { 'color': color_lib[idx], 'name': clade }
            if idx < 23:
                idx += 1
            else:
                idx = 0

        # Config for the plot
        config = {
            'cpswitch_c_active': False,
            'id': 'mirtrace_contamination_check_plot',
            'title': 'miRTrace: Contamination Check',
            'ylab': '# miRNA detected',
            'cpswitch_counts_label': 'Number of detected miRNA'
        }

        return bargraph.plot(self.contamination_data, keys, config)


    # miRTrace Read Length Distribution
    def mirtrace_complexity_plot(self):
        """ Generate the miRTrace miRNA Complexity Plot"""

        data = dict()
        for s_name in self.complexity_data:
            try:
                data[s_name] = {int(self.complexity_data[s_name][d]) : int(d) for d in self.complexity_data[s_name]}
            except KeyError:
                pass
        if len(data) == 0:
            log.debug('No valid data for miRNA complexity')
            return None

        config = {
            'id': 'mirtrace_complexity_plot',
            'title': 'miRTrace: miRNA Complexity Plot',
            'ylab': 'Distinct miRNA Count',
            'xlab': 'Number of Sequencing Reads',
            'ymin': 0,
            'xmin': 1,
            'xDecimals': False,
            'tt_label': '<b>Number of Sequencing Reads {point.x}</b>: {point.y} Distinct miRNA Count',
        }

        return linegraph.plot(data, config)
